/***************************************************************************
                         filebrowse.h  -  description
                             -------------------
    project              : GTagger
    version              : 0.1
    begin                : Sun Jan 10 2002
    lastchange           : Sun Jan 27 2002
    copyright            : (C) 2003 by Marcel Karras
    email                : toka@freebits.de
    homepage             : http://www.freebits.de
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#ifndef __FILEBROWSE_H
#define __FILEBROWSE_H
#include <gtkmm/scrolledwindow.h>
#include <gtkmm/treemodel.h>
#include <gtkmm/treeview.h>
#include <gtkmm/liststore.h>
#include <gtkmm/entry.h>
#include <gdkmm/pixbuf.h>
#include <list>

/**
 * Lists all files in a selected directory with additional properties.
 */

class FileBrowse : public Gtk::ScrolledWindow
{
    /**
     * built-in-class for column representation
     */
    class ModelColumns : public Gtk::TreeModel::ColumnRecord
    {
      public:		
        Gtk::TreeModelColumn<Glib::RefPtr<Gdk::Pixbuf> > m_c_thumb;
        Gtk::TreeModelColumn<Glib::ustring>              m_c_name;
	  
        ModelColumns()
        { add(m_c_thumb); add(m_c_name); }
    };
    
  
  public:
    typedef Glib::RefPtr<Gtk::ListStore> PtrListStore;
    /**
     * type for several mime types in order to define a mime icon
     */
    enum IconType{
      UNKNOWN_MIME,
      SOUND_MIME
    };
    
    /**
     * Constructor
     */
    FileBrowse();
    /**
     * Destructor
     */
    
    virtual ~FileBrowse();
    /**
     * append a row to the list
     * @param str - filename
     * @param type - MIME type
     */
    void appendRow(const Glib::ustring& str, IconType type);
    /**
     * remove the selected row from list
     */
    void del_selected_row();
    /**
     * remove the row pointed to
     * @param i - TreeModel::iterator pointing to the row
     */
    void del_row(Gtk::TreeModel::iterator i);
    /**
     * search a filename and return result
     * @param s - filename
     * @return (bool) - true if found else false
     */
    bool searchName(const Glib::ustring& s);
    /**
     * get an iterator pointing to a given filename
     * @param s - filename
     * @return (TreeModel::iterator) - pointer to row
     */
    Gtk::TreeModel::iterator getName(const Glib::ustring& s);
    /**
     * rename the filename of the current row
     * @param s - filename to be set
     */
    void rename_selected_row(const Glib::ustring& s);
    /**
     * clear the whole list
     */
    void clear();
    /**
     * return the filename of the current selected row
     * @return (ustring) - filename
     */
    Glib::ustring getCurrentRowContent();
    /**
     * connect a slot to a function responsable to a row_change event
     * @param slot_activated - SigC::slot
     */
    void connect_row_changed(const SigC::Slot0<void>& slot_activated);
  
  protected:	   
    /**
     * force the emission of a row change signal
     * @see connect_row_changed
     */
    void activate_row_changed();
    /**
     * default sort function for list
     * @param lhs - left side of boolean comparison
     * @param rhs - right side of boolean comparison
     * @return (int) - -1(lower), 0(equal) or 1(greater)
     */
    int default_sort_func(const Gtk::TreeModel::iterator& lhs, 
                          const Gtk::TreeModel::iterator& rhs);
  
  private:
    Gtk::TreeView                          w_treeview;
    ModelColumns                           m_columns;  
    PtrListStore                           p_treemodel;
    std::list< Glib::RefPtr<Gdk::Pixbuf> > icons;
    SigC::Signal0<void>                    signal_change_row;
};


#endif
