/***************************************************************************
                         fileprefs.h  -  description
                             -------------------
    project              : GTagger
    version              : 0.1
    begin                : Sun Jan 10 2002
    lastchange           : Sun Jan 27 2002
    copyright            : (C) 2003 by Marcel Karras
    email                : toka@freebits.de
    homepage             : http://www.freebits.de
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/
 
#ifndef __FILEPREFS_H
#define __FILEPREFS_H
#include <gtkmm/scrolledwindow.h>
#include <gtkmm/table.h>
#include <gtkmm/frame.h>
#include <gtkmm/alignment.h>
#include <gtkmm/label.h>
#include <gtkmm/entry.h>
#include <gtkmm/combo.h>
#include <gtkmm/eventbox.h>

/**
 * Frame showing file preferences and tag information
 */

class FilePrefs : public Gtk::Frame
{
  public:
    /**
     * Constructor
     */
    FilePrefs();
    /**
     * Destructor
     */
    virtual ~FilePrefs();
    /**
     * clear all Gtk::Entries
     */
    void reset_all_entries();
    /**
     * set the text of the filename Gtk::Entry
     * @param str - text to be set
     */
    void set_e_filename_text(const Glib::ustring& str);
    /**
     * set the text of the filesize Gtk::Entry
     * @param str - s to be set
     */
    void set_e_filesize_text(const Glib::ustring& str);
    /**
     * set the text of the tag version Gtk::Entry
     * @param str - text to be set
     */
    void set_e_tag_version_text(const Glib::ustring& str);
    /**
     * set the text of the tag title Gtk::Entry
     * @param str - text to be set
     */
    void set_e_tag_title_text(const Glib::ustring& str);
    /**
     * set the text of the tag artist Gtk::Entry
     * @param str - text to be set
     */
    void set_e_tag_artist_text(const Glib::ustring& str);
    /**
     * set the text of the tag album Gtk::Entry
     * @param str - text to be set
     */
    void set_e_tag_album_text(const Glib::ustring& str);
    /**
     * set the text of the tag track Gtk::Entry
     * @param str - text to be set
     */
    void set_e_tag_track_text(const Glib::ustring& str);
    /**
     * set the text of the suggested filename Gtk::Entry
     * @param str - text to be set
     */
    void set_e_prop_filename_text(const Glib::ustring& str);
    /**
     * set the text of the genre Gtk::Combo
     * @param str - text to be set
     */
    void set_e_tag_genre_text(const std::string& str);
    /**
     * set all tag Gtk::Entry's translucent
     * @param sens - true if translucent else false
     */
    void set_e_tags_sensitive(bool sens);
  
    /**
     * return the text of tag title Gtk::Entry
     * @return (ustring) - text
     */
    Glib::ustring get_e_tag_title_text() const;
    /**
     * return the text of tag artist Gtk::Entry
     * @return (ustring) - text
     */
    Glib::ustring get_e_tag_artist_text() const;
    /**
     * return the text of tag album Gtk::Entry
     * @return (ustring) - text
     */
    Glib::ustring get_e_tag_album_text() const;
    /**
     * return the number of tag track Gtk::Entry
     * @return (unsigned char) - track number
     */
    unsigned char get_e_tag_track_text() const;
    /**
     * return the text of the suggested filename
     * @return (ustring) - text
     */
    Glib::ustring get_e_prop_filename_text() const;
    /**
     * return the text of tag genre Gtk::Combo
     * @return (string) - text
     */
    std::string   get_e_tag_genre_text() const;
  
    /**
     * connect a slot to a function which handles clicked() on file rename
     * @param slot_activated - SigC::slot
     */
    void connect_rename_clicked(const SigC::Slot0<void>& slot_activated);
    /**
     * connect a slot to a function which handles text inserts within tag field
     * @param slot_activated - SigC::slot
     */
    void connect_tag_inserted(const SigC::Slot1<bool,GdkEventKey*>& 
                              slot_activated);
  
  protected:
    /**
     * force emission of the rename file clicked() signal
     */
    void activate_rename_clicked();
    /**
     * force emission of the insert tag signal
     */
    bool activate_tag_inserted(GdkEventKey* event);

  private:
    Gtk::ScrolledWindow	w_global_scrollwin;
    Gtk::Table	w_scrollwin_table;
    Gtk::Table	w_file_table;
    Gtk::Table	w_tag_table;
    Gtk::Table	w_name_table;
    Gtk::Frame	w_file_frame;
    Gtk::Frame	w_tag_frame;
    Gtk::Frame	w_name_frame;
    Gtk::Alignment w_label_filename;
    Gtk::Alignment w_label_filesize;
    Gtk::Alignment w_label_tag_version;
    Gtk::Alignment w_label_tag_title;
    Gtk::Alignment w_label_tag_artist;
    Gtk::Alignment w_label_tag_album;
    Gtk::Alignment w_label_tag_track;
    Gtk::Alignment w_label_tag_genre;
    Gtk::Label w_l_filename;
    Gtk::Label w_l_filesize;
    Gtk::Label w_l_tag_version;
    Gtk::Label w_l_tag_title;
    Gtk::Label w_l_tag_artist;
    Gtk::Label w_l_tag_album;
    Gtk::Label w_l_tag_track;
    Gtk::Label w_l_tag_genre;
    Gtk::Entry w_e_filename;
    Gtk::Entry w_e_filesize;
    Gtk::Entry w_e_tag_version;
    Gtk::Entry w_e_tag_title;
    Gtk::Entry w_e_tag_artist;
    Gtk::Entry w_e_tag_album;
    Gtk::Entry w_e_tag_track;
    Gtk::Combo w_e_tag_genre;
    Gtk::Entry w_e_prop_filename;
    SigC::Signal0<void> signal_rename_clicked;
    SigC::Signal1<bool,GdkEventKey*> signal_tag_inserted;
};

#endif
