/***************************************************************************
                    filesystemexception.h  -  description
                             -------------------
    project              : GTagger
    version              : 0.1
    begin                : Sun Jan 10 2002
    lastchange           : Sun Jan 27 2002
    copyright            : (C) 2003 by Marcel Karras
    email                : toka@freebits.de
    homepage             : http://www.freebits.de
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/
 
#ifndef __FILESYSTEMEXCEPTION_H
#define __FILESYSTEMEXCEPTION_H
#include <exception>
#include <glibmm/ustring.h>

/**
 * exception class which displays an error in an ErrorDialog
 */
class FileSystemException : public std::exception
{
  public:
    /**
     * error types specifying the appropriate error message
     */
    enum ErrorTypes{
      /** directory error */
      FS_STAT_DIR,
      /** file error */
      FS_STAT_FILE,
      /** file rename error */
      FS_RENAME,
      /** file delete error */
      FS_DELETE
    };
	
    /**
     * Constructor
     * @param err - error type
     * @param add_msg - additional message to be appended
     */
    FileSystemException(const ErrorTypes& err, const Glib::ustring& add_msg="")
      throw();
    /**
     * Destructor
     */
    ~FileSystemException()
      throw();

    /**
     * pop up a dialog and display the error message
     */    
    void showErrorDialog() const
      throw();
  
  private:
    ErrorTypes _error;
    Glib::ustring _addmsg;
	  
};

 
#endif
