/***************************************************************************
                         menubar.h  -  description
                             -------------------
    project              : GTagger
    version              : 0.1
    begin                : Sun Jan 10 2002
    lastchange           : Sun Jan 27 2002
    copyright            : (C) 2003 by Marcel Karras
    email                : toka@freebits.de
    homepage             : http://www.freebits.de
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#ifndef __MENUBAR_H
#define __MENUBAR_H
#include <gtkmm/menubar.h>
#include <gtkmm/menu.h>
#include <gtkmm/image.h>

/**
 * Menubar representation of GTagger
 */

class MenuBar : public Gtk::MenuBar
{
  public:
    /**
     * Constructor
     */
    MenuBar();
    /**
     * Default Destructor
     */
    virtual ~MenuBar();
	
    /**
     * connect a slot to the "open file" menu entry
     * @param slot_activated - SigC::slot
     */
    void connect_file_open(const SigC::Slot0<void>& slot_activated);
    /**
     * connect a slot to the "rename file" menu entry
     * @param slot_activated - SigC::slot
     */
    void connect_file_rename(const SigC::Slot0<void>& slot_activated);
    /**
     * connect a slot to the "delete file" menu entry
     * @param slot_activated - SigC::slot
     */
    void connect_file_delete(const SigC::Slot0<void>& slot_activated);
    /**
     * connect a slot to the "add tag" menu entry
     * @param slot_activated - SigC::slot
     */
    void connect_tag_add(const SigC::Slot0<void>& slot_activated);
    /**
     * connect a slot to the "delete tag" menu entry
     * @param slot_activated - SigC::slot
     */
    void connect_tag_del(const SigC::Slot0<void>& slot_activated);
    /**
     * connect a slot to the "save tag" menu entry
     * @param slot_activated - SigC::slot
     */
    void connect_tag_save(const SigC::Slot0<void>& slot_activated);
    /**
     * connect a slot to the id3tag version toggle menu entry
     * @param slot_activated - SigC::slot to a function with int param (version)
     */
    void connect_tag_version_changed(
                     const SigC::Slot1<void,int>& slot_activated);
    /**
     * connect a slot to the "reload directory" menu entry
     * @param slot_activated - SigC::slot
     */
    void connect_reload_pressed(const SigC::Slot0<void>& slot_activated);
    /**
     * connect a slot to the "save all" menu entry
     * @param slot_activated - SigC::slot
     */
    void connect_save_all_pressed(const SigC::Slot0<void>& slot_activated);
  
    /**
     * make the rename entry translucent
     * @param sens - true if translucent else false
     */
    void set_rename_sensitive(bool sens);
    /**
     * make the delete entry translucent
     * @param sens - true if translucent else false
     */
    void set_delete_sensitive(bool sens);
    /**
     * make the save tag entry translucent
     * @param sens - true if translucent else false
     */
    void set_tag_save_sensitive(bool sens);
    /**
     * make the add tag entry translucent
     * @param sens - true if translucent else false
     */
    void set_tag_add_sensitive(bool sens);
    /**
     * make the delete tag entry translucent
     * @param sens - true if translucent else false
     */
    void set_tag_del_sensitive(bool sens);
    /**
     * make the delete entry translucent
     * @param sens - true if translucent else false
     */
    void set_save_all_sensitive(bool sens);
    /**
     * set the toggle entries to a specific one
     * @param version - 1=id3v1, 2=id3v2, 3=id3v1&2
     */
   
    void set_version_button(int version);
    /**
     * get the current selected version toggle button
     * @return (int) - 1=id3v1, 2=id3v2, 3=id3v1&2
     */
    int get_version_button() const;
  
  private:
    Gtk::Menu w_menu_file;
    Gtk::Menu w_menu_edit;
    Gtk::Menu w_menu_help;
    Gtk::Image w_rename_image;
    Gtk::Image w_info_image;  
    Gtk::Image w_tag_add_image;
    Gtk::Image w_tag_del_image;
    Gtk::Image w_tag_save_image;
    Gtk::Image w_delete_image;
    Gtk::Image w_reload_image;
    Gtk::Image w_save_all_image;
  
    SigC::Signal0<void>	signal_file_open;
    SigC::Signal0<void>	signal_file_rename;
    SigC::Signal0<void>	signal_file_delete;
    SigC::Signal0<void>	signal_tag_add;
    SigC::Signal0<void> signal_tag_del;
    SigC::Signal0<void> signal_tag_save;
    SigC::Signal1<void,int> signal_tag_version_changed;
    SigC::Signal0<void> signal_reload_pressed;
    SigC::Signal0<void> signal_save_all_pressed;
  
  protected:
    /**
     * function called on clicked()-event of "info" button (ctrl+i)
     */
    void on_help_info_clicked();
    /**
     * function called on clicked()-event of "open" button (ctrl+o)
     */
    void activate_file_open();		
    /**
     * function called on clicked()-event of "rename" button (ctrl+r)
     */
    void activate_file_rename();
    /**
     * function called on clicked()-event of "delete file" button (ctrl+d)
     */
    void activate_file_delete();
    /**
     * function called on clicked()-event of "add tag" button (ctrl+ins)
     */
    void activate_tag_add();
    /**
     * function called on clicked()-event of "delete tag" button (ctrl+del)
     */
    void activate_tag_del();
    /**
     * function called on clicked()-event of "save tag" button (ctrl+s)
     */
    void activate_tag_save();
    /**
     * function called on clicked()-event of toggle buttons (ctrl+1/2/3)
     */
    void activate_tag_version_changed();
    /**
     * function called on clicked()-event of "reload list" button (ctrl+a)
     */
    void activate_reload_pressed();
    /**
     * function called on clicked()-event of "save all" button (ctrl+w)
     */
    void activate_save_all_pressed();
  	
 };

#endif
