/***************************************************************************
                         status.h  -  description
                             -------------------
    project              : GTagger
    version              : 0.1
    begin                : Sun Jan 20 2002
    lastchange           : Sun Jan 27 2002
    copyright            : (C) 2003 by Marcel Karras
    email                : toka@freebits.de
    homepage             : http://www.freebits.de
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/
 
#ifndef __STATUS_H
#define __STATUS_H
#include <gtkmm/table.h>
#include <gtkmm/statusbar.h>
#include <glibmm/ustring.h>
 
/**
 * This class embodies a statusbarset with simple timeout capabilities.
 * Every registered timeout will be unregistered and overwritten if a
 * new one is to be connected.
 */
class Status : public Gtk::Table
{
  public:
    
    Status();
    virtual ~Status();
    
    /** 
     * Set a text to a specific statusbar and install it for a given time
     * or forever.
     * @param s - text to be set
     * @param statusbar - 0 (left bar) | 1 (right bar)
     * @param timeout - timeout in ms (remove text afterwards)
     */
    void set_text(const Glib::ustring& s, int statusbar, int timeout=0);

    /**
     * Remove any installed text and connected timeout.
     * @param statusbar - 0 (left bar) | 1 (right bar)
     */
    void remove_text(int statusbar);
  
    /**
     * Check if the statusbar contains text to be timeouted.
     * @param statusbar - 0 (left bar) | 1 (right bar)
     * @return bool - true if timeout installed, else false
     */
    bool has_timeout(int statusbar);
    
  protected:
    /**
     * This callback is called if you've set a timeout via set_text().
     * @param statusbar - 0 (left bar) | 1 (right bar)
     * @return bool - false (timeout stop) | true (timeout repeat)
     */
    bool timer_callback(int statusbar);
  
    /**
     * Disconnect any timer on a given statusbar
     * @param statusbar - 0 (left bar) | 1 (right bar)
     */
    void timer_disconnect(int statusbar);

  private:
    
    Gtk::Statusbar w_left_bar;
    Gtk::Statusbar w_right_bar;
    // connection handler used to disconnect a timeout
    SigC::Connection conn_left;
    SigC::Connection conn_right;
};
 
#endif
