/***************************************************************************
                         toolbar.h  -  description
                             -------------------
    project              : GTagger
    version              : 0.1
    begin                : Sun Jan 14 2002
    lastchange           : Sun Jan 27 2002
    copyright            : (C) 2003 by Marcel Karras
    email                : toka@freebits.de
    homepage             : http://www.freebits.de
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#ifndef __TOOLBAR_H
#define __TOOLBAR_H
#include <gtkmm/toolbar.h>
#include <gtkmm/button.h>
#include <gtkmm/image.h>
#include <gtkmm/alignment.h>
#include <gtkmm/label.h>
#include <gtkmm/box.h>
#include <gtkmm/combo.h>
#include <gtkmm/radiobutton.h>
#include <gtkmm/accelgroup.h>

/**
 * Toolbar with implemented buttons, icons and entries
 */
class ToolBar : public Gtk::Toolbar
{
  public:
    /**
     * Constructor
     */
    ToolBar();
    /**
     * Destructor
     */
    virtual ~ToolBar();

    /**
     * set the "delete file" button translucent
     * @param sensitive - true=translucent, false=non-translucent
     */
    void b_delete_file_sensitive(bool sensitive);
    /**
     * set the "rename file" button translucent
     * @param sensitive - true=translucent, false=non-translucent
     */
    void b_rename_file_sensitive(bool sensitive);
    /**
     * set the "save tag" button translucent
     * @param sensitive - true=translucent, false=non-translucent
     */
    void b_tag_file_sensitive(bool sensitive);
    /**
     * set the "add file" button translucent
     * @param sensitive - true=translucent, false=non-translucent
     */
    void b_tag_add_file_sensitive(bool sensitive);
    /**
     * set the "delete tag" button translucent
     * @param sensitive - true=translucent, false=non-translucent
     */
    void b_tag_del_file_sensitive(bool sensitive);
    /**
     * set the "save all" button translucent
     * @param sensitive - true=translucent, false=non-translucent
     */
    void b_save_all_sensitive(bool sensitive);
    /**
     * set the version toggle button
     * @param version - 1=id3v1, 2=id3v2, 3=id3v1&2
     */
    void set_version_button(int version);
    /**
     * return the currently selected version 
     * @return (int) - 1=id3v1, 2=id3v2, 3=id3v1&2
     */
    int get_version_button() const;
  
    /**
     * connect a slot to a function handling a delete clicked() event
     * @param slot_activated - SigC::slot
     */
    void connect_delete_pressed(const SigC::Slot0<void>& slot_activated);
    /**
     * connect a slot to a function handling a open clicked() event
     * @param slot_activated - SigC::slot
     */
    void connect_open_pressed(const SigC::Slot0<void>& slot_activated);
    /**
     * connect a slot to a function handling a rename clicked() event
     * @param slot_activated - SigC::slot
     */
    void connect_rename_pressed(const SigC::Slot0<void>& slot_activated);
    /**
     * connect a slot to a function handling a save tag clicked() event
     * @param slot_activated - SigC::slot
     */
    void connect_tag_pressed(const SigC::Slot0<void>& slot_activated);
    /**
     * connect a slot to a function handling an add tag clicked() event
     * @param slot_activated - SigC::slot
     */
    void connect_tag_add_pressed(const SigC::Slot0<void>& slot_activated);
    /**
     * connect a slot to a function handling a delete tag clicked() event
     * @param slot_activated - SigC::slot
     */
    void connect_tag_del_pressed(const SigC::Slot0<void>& slot_activated);
    /**
     * connect a slot to a function handling a version toggle() event
     * @param slot_activated - SigC::slot
     */
    void connect_tag_version_changed(const SigC::Slot1<void,int>& 
                                     slot_activated);
    /**
     * connect a slot to a function handling a reload clicked() event
     * @param slot_activated - SigC::slot
     */
    void connect_reload_pressed(const SigC::Slot0<void>& slot_activated);
    /**
     * connect a slot to a function handling a save all clicked() event
     * @param slot_activated - SigC::slot
     */
    void connect_save_all_pressed(const SigC::Slot0<void>& slot_activated);
   
  protected:
    /**
     * force the emission of delete clicked() signal
     */
    void activate_delete_pressed();
    /**
     * force the emission of open clicked() signal
     */
    void activate_open_pressed();
    /**
     * force the emission of rename clicked() signal
     */
    void activate_rename_pressed();
    /**
     * force the emission of save tag clicked() signal
     */
    void activate_tag_pressed();
    /**
     * force the emission of add tag clicked() signal
     */
    void activate_tag_add_pressed();
    /**
     * force the emission of delete tag clicked() signal
     */
    void activate_tag_del_pressed();
    /**
     * force the emission of version toggle() signal
     */
    void activate_tag_version_changed();
    /**
     * force the emission of reload clicked() signal
     */
    void activate_reload_pressed();
    /**
     * force the emission of save all clicked() signal
     */
    void activate_save_all_pressed();
  
  private:
    Gtk::Button	b_delete_file;
    Gtk::Button	b_rename_file;
    Gtk::Button	b_tag_file;
    Gtk::Button	b_save_all;
    Gtk::Button	b_tag_add_file;
    Gtk::Button	b_tag_del_file;
    Gtk::Button	b_reload_list;
    Gtk::Image b_delete_file_image; 
    Gtk::Image b_rename_file_image; 
    Gtk::Image b_tag_file_image;
    Gtk::Image b_save_all_image;
    Gtk::Image b_tag_add_file_image;
    Gtk::Image b_tag_del_file_image;
    Gtk::Image b_id3v1_image;	
    Gtk::Image b_id3v2_image;
    Gtk::Image b_id3v12_image;
    Gtk::Image b_reload_image;
  	
    Gtk::RadioButton* p_id3v1;
    Gtk::RadioButton* p_id3v2;
    Gtk::RadioButton* p_id3v12;
  
    SigC::Signal0<void> signal_delete_pressed;
    SigC::Signal0<void> signal_open_pressed;
    SigC::Signal0<void> signal_rename_pressed;
    SigC::Signal0<void> signal_tag_pressed;
    SigC::Signal0<void> signal_tag_add_pressed;
    SigC::Signal0<void> signal_tag_del_pressed;
    SigC::Signal1<void,int> signal_tag_version_changed;
    SigC::Signal0<void>	signal_reload_pressed;
    SigC::Signal0<void>	signal_save_all_pressed;
};

#endif
