/***************************************************************************
                      windowtoplevel.h  -  description
                             -------------------
    project              : GTagger
    version              : 0.1
    begin                : Sun Jan 10 2002
    lastchange           : Sun Jan 27 2002
    copyright            : (C) 2003 by Marcel Karras
    email                : toka@freebits.de
    homepage             : http://www.freebits.de
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#ifndef __WINDOWTOPLEVEL_H
#define __WINDOWTOPLEVEL_H
#include <gtkmm/window.h>
#include <gtkmm/table.h>
#include <gtkmm/paned.h>
#include <gtkmm/box.h>
#include <gtkmm/handlebox.h>
#include <gtkmm/statusbar.h>
#include <glibmm/ustring.h>
#include "menubar.h"
#include "toolbar.h"
#include "fileprefs.h"
#include "filebrowse.h"
#include "filesystemexception.h"
#include "status.h"

/**
 * top-level class which embodies the whole application
 */
class WindowToplevel : public Gtk::Window
{
  public:
    /**
     * Constructor
     */
    WindowToplevel();
    /**
     * Destructor
     */
    virtual ~WindowToplevel();
  
  protected:
    /**
     * does various stuff if no selection made (i.e. deselecting, resetting)
     */
    void no_selection();
    /**
     * open the directory selection dialog
     * @exception FileSystemException
     */
    void s_button_open_clicked()
      throw(FileSystemException);
    /**
     * reload the file properties
     */
    void reload_properties();
    /**
     * rename the selected file
     */
    void s_button_rename_clicked();
    /**
     * delete the selected file
     */
    void s_button_delete_clicked();
    /**
     * create ID3Tag
     */
    void s_button_tag_add_clicked();
    /**
     * delete ID3Tag
     */
    void s_button_tag_del_clicked();
    /**
     * save ID3Tag
     */
    void s_button_tag_clicked();
    /**
     * set ID3Tag version
     * @param source - 0=menu, 1=toolbar (internal usage)
     */
    void set_tag_version(int source);
    /**
     * reload the file list
     */
    void s_reload_pressed();
    /**
     * set the file count within statusbar
     * @param f - file count
     */
    void set_files(unsigned long f);
    /**
     * handle inputs from tag entries
     * @param event - which key event occured
     * @return (bool) - true = successful handling else false
     */
    bool tag_changed(GdkEventKey* event);
    /**
     * refresh the suggested filename if text inserted
     */
    void refresh_proposal_filename();
    /**
     * save tag information and rename file
     */
    void s_save_all_pressed();
    
  private:
    Gtk::Table    w_toplevel_table;
    Gtk::VPaned   w_toplevel_vpaned;
    MenuBar       w_menu_bar;
    ToolBar       w_tool_bar;
    FilePrefs     w_file_prefs;
    FileBrowse    w_file_browse;
    Status        w_statusbar;
    Glib::ustring s_current_dir;
    Glib::ustring s_last_file;
    Glib::ustring s_current_tag_version;
    unsigned long all_files;
};

#endif
