/***************************************************************************
                        filebrowse.cc  -  description
                             -------------------
    project              : GTagger
    version              : 0.1
    begin                : Sun Jan 10 2002
    lastchange           : Sun Jan 27 2002
    copyright            : (C) 2003 by Marcel Karras
    email                : toka@freebits.de
    homepage             : http://www.freebits.de
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#define  _(x) gettext(x)
#include <libintl.h>
#include "filebrowse.h"
#include "pixmaps.h"
#include <gtkmm/stock.h>
#include <string>
#include <iostream>

FileBrowse::FileBrowse()
{
  set_policy(Gtk::POLICY_AUTOMATIC,Gtk::POLICY_AUTOMATIC);
  p_treemodel = Gtk::ListStore::create(m_columns);
  p_treemodel->set_default_sort_func(SigC::slot(*this,
                                     &FileBrowse::default_sort_func));
  w_treeview.set_model(p_treemodel);
  w_treeview.set_reorderable(true);
  w_treeview.append_column(Glib::locale_to_utf8(_("Type")), 
                           m_columns.m_c_thumb);
  w_treeview.append_column(Glib::locale_to_utf8(_("Filename")), 
                           m_columns.m_c_name);
  w_treeview.get_selection()->signal_changed().connect(
             SigC::slot(*this,&FileBrowse::activate_row_changed));
  icons.push_back(Gdk::Pixbuf::create_from_inline(sizeof(sound_inline),
                                                  sound_inline));
  icons.push_back(Gdk::Pixbuf::create_from_inline(sizeof(unknown_inline),
                                                  unknown_inline));
  add(w_treeview);
  show_all();
}
	
FileBrowse::~FileBrowse()
{
}

void FileBrowse::appendRow(const Glib::ustring& str, IconType type)
{
  Gtk::TreeModel::Row row = *(p_treemodel->append());
  row[m_columns.m_c_name] = str;
  if(type == SOUND_MIME)
    row[m_columns.m_c_thumb] = icons.front();
  else
    row[m_columns.m_c_thumb] = icons.back();	
}

void FileBrowse::clear()
{
  p_treemodel->clear();
}

void FileBrowse::activate_row_changed()
{
  signal_change_row();
}

void FileBrowse::connect_row_changed(const SigC::Slot0<void>& slot_activated)
{
  signal_change_row.connect(slot_activated);
}

Glib::ustring FileBrowse::getCurrentRowContent()
{
  Gtk::TreeModel::Row row = *(w_treeview.get_selection()->get_selected());
  if(row)
    return row[m_columns.m_c_name];
  return Glib::ustring("");
}

void FileBrowse::del_selected_row()
{
  p_treemodel->erase(w_treeview.get_selection()->get_selected());
}

void FileBrowse::del_row(Gtk::TreeModel::iterator i)
{
  p_treemodel->erase(i);
}

bool FileBrowse::searchName(const Glib::ustring& s)
{
  Gtk::TreeModel::iterator it = p_treemodel->children().begin();
  for(uint i = 0; i < p_treemodel->children().size(); i++)
  {
    if((Gtk::TreeModel::Row(*it))[m_columns.m_c_name] == s)
      return true;
    it++;
  }	
  return false;
}

Gtk::TreeModel::iterator FileBrowse::getName(const Glib::ustring& s)
{
  Gtk::TreeModel::iterator it = p_treemodel->children().begin();
  for(uint i = 0; i < p_treemodel->children().size(); i++)
  {
    if((Gtk::TreeModel::Row(*it))[m_columns.m_c_name] == s)
      return it;
    it++;
  }	
  return Gtk::TreeModel::iterator(NULL);
}

void FileBrowse::rename_selected_row(const Glib::ustring& s)
{
  Gtk::TreeModel::Path path = p_treemodel->get_path(
                              w_treeview.get_selection()->get_selected());
  Gtk::TreeModel::Row row = *(w_treeview.get_selection()->get_selected());
  row[m_columns.m_c_name] = s;
  p_treemodel->row_changed(path,Gtk::TreeModel::iterator(row));
  // update icon if required
  if(s.size()>2)
  {
    Glib::ustring st = Glib::ustring(s,s.size()-3,3);
    if( (st == "mp3") || (st == "mP3") || (st == "Mp3") || (st == "MP3")
    ||  (st == "ogg") || (st == "Ogg") || (st == "OGg") || (st == "OgG")
    ||  (st == "oGG") || (st == "ogG") || (st == "oGg")) 
      row[m_columns.m_c_thumb] = icons.front();
    else
      row[m_columns.m_c_thumb] = icons.back();
  }
  else
    row[m_columns.m_c_thumb] = icons.back();
}

int FileBrowse::default_sort_func(const Gtk::TreeModel::iterator& lhs, 
                                  const Gtk::TreeModel::iterator& rhs)
{
  const std::string lhs_key(Glib::locale_from_utf8((*lhs)[m_columns.m_c_name]));
  const std::string rhs_key(Glib::locale_from_utf8((*rhs)[m_columns.m_c_name]));
  return lhs_key.compare(rhs_key);
}
