/***************************************************************************
                         fileprefs.cc  -  description
                             -------------------
    project              : GTagger
    version              : 0.1
    begin                : Sun Jan 10 2002
    lastchange           : Sun Jan 27 2002
    copyright            : (C) 2003 by Marcel Karras
    email                : toka@freebits.de
    homepage             : http://www.freebits.de
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#define  _(x) gettext(x)
#include <libintl.h>
#include "fileprefs.h"
#include "genre.h"
#include <gtkmm/viewport.h>
#include <glibmm/listhandle.h>
#include <glibmm/ustring.h>
#include <iostream>
#include <sstream>
#include <vector>
#include <algorithm>

inline short int convstring(const std::string &s)
{
  if(s.empty())
    return 0;
  std::istringstream in(s);
  short int n;
  in >> n;
  return n;
}

FilePrefs::FilePrefs()
: w_scrollwin_table(1,3),
  w_file_table(2,2),
  w_tag_table(4,5),
  w_name_table(1,1),
  w_file_frame(Glib::locale_to_utf8(_("File Information"))),
  w_tag_frame(Glib::locale_to_utf8(_("Tag Attributes"))),
  w_name_frame(Glib::locale_to_utf8(_("Suggested Filename"))),
  w_label_filename(Gtk::ALIGN_RIGHT, Gtk::ALIGN_CENTER, 0.0, 0.0),
  w_label_filesize(Gtk::ALIGN_RIGHT, Gtk::ALIGN_CENTER, 0.0, 0.0),
  w_label_tag_version(Gtk::ALIGN_RIGHT, Gtk::ALIGN_CENTER, 0.0, 0.0),
  w_label_tag_title(Gtk::ALIGN_RIGHT, Gtk::ALIGN_CENTER, 0.0, 0.0),
  w_label_tag_artist(Gtk::ALIGN_RIGHT, Gtk::ALIGN_CENTER, 0.0, 0.0),
  w_label_tag_album(Gtk::ALIGN_RIGHT, Gtk::ALIGN_CENTER, 0.0, 0.0),
  w_label_tag_track(Gtk::ALIGN_RIGHT, Gtk::ALIGN_CENTER, 0.0, 0.0),
  w_label_tag_genre(Gtk::ALIGN_RIGHT, Gtk::ALIGN_CENTER, 0.0, 0.0),
  w_l_filename(Glib::locale_to_utf8(_("Filename :"))),
  w_l_filesize(Glib::locale_to_utf8(_("Size :"))),
  w_l_tag_version(Glib::locale_to_utf8(_("ID3 Version :"))),
  w_l_tag_title(Glib::locale_to_utf8(_("Title :"))),
  w_l_tag_artist(Glib::locale_to_utf8(_("Artist :"))),
  w_l_tag_album(Glib::locale_to_utf8(_("Album :"))),
  w_l_tag_track(Glib::locale_to_utf8(_("Track :"))),
  w_l_tag_genre(Glib::locale_to_utf8(_("Genre :")))
{	
  w_e_tag_title.set_events(Gdk::KEY_PRESS_MASK|Gdk::KEY_RELEASE_MASK);
  std::vector<std::string> genre_list;
  genre_list.push_back(std::string("Unknown"));
  for(uint i=0; i < 148; i++)
  genre_list.push_back(std::string(genre_table[i]));
  sort(genre_list.begin(),genre_list.end());
  w_e_tag_genre.set_popdown_strings(genre_list);
  w_e_tag_genre.get_entry()->set_editable(false);
  w_e_filename.set_editable(false);
  w_e_filesize.set_editable(false);
  w_e_tag_track.set_max_length(3);
  w_e_tag_track.set_width_chars(4);
  w_e_tag_version.set_editable(false);
  
  w_file_table.set_border_width(5);
  w_tag_table.set_border_width(5);
  w_name_table.set_border_width(5);
  
  w_label_filename.add(w_l_filename);
  w_label_filesize.add(w_l_filesize);
  w_label_tag_version.add(w_l_tag_version);
  w_label_tag_title.add(w_l_tag_title);
  w_label_tag_artist.add(w_l_tag_artist);
  w_label_tag_album.add(w_l_tag_album);
  w_label_tag_track.add(w_l_tag_track);
  w_label_tag_genre.add(w_l_tag_genre);
  
  w_file_table.attach(w_label_filename,0,1,0,1,Gtk::FILL,Gtk::FILL,5);	
  w_file_table.attach(w_e_filename,1,2,0,1,Gtk::FILL|Gtk::EXPAND,Gtk::FILL,5);
  w_file_table.attach(w_label_filesize,0,1,1,2,Gtk::FILL,Gtk::FILL,5);
  w_file_table.attach(w_e_filesize,1,2,1,2,Gtk::FILL|Gtk::EXPAND,Gtk::FILL,5);
  
  w_tag_table.attach(w_label_tag_version,0,1,0,1,Gtk::FILL,Gtk::FILL,5);
  w_tag_table.attach(w_e_tag_version,1,4,0,1,Gtk::FILL|Gtk::EXPAND,Gtk::FILL,5);
  w_tag_table.attach(w_label_tag_title,0,1,1,2,Gtk::FILL,Gtk::FILL,5);
  w_tag_table.attach(w_e_tag_title,1,4,1,2,Gtk::FILL|Gtk::EXPAND,Gtk::FILL,5);
  w_tag_table.attach(w_label_tag_artist,0,1,2,3,Gtk::FILL,Gtk::FILL,5);
  w_tag_table.attach(w_e_tag_artist,1,4,2,3,Gtk::FILL|Gtk::EXPAND,Gtk::FILL,5);
  w_tag_table.attach(w_label_tag_album,0,1,3,4,Gtk::FILL,Gtk::FILL,5);
  w_tag_table.attach(w_e_tag_album,1,4,3,4,Gtk::FILL|Gtk::EXPAND,Gtk::FILL,5);
  w_tag_table.attach(w_label_tag_track,0,1,4,5,Gtk::FILL,Gtk::FILL,5);
  w_tag_table.attach(w_e_tag_track,1,2,4,5,Gtk::FILL,Gtk::FILL,5);
  w_tag_table.attach(w_label_tag_genre,2,3,4,5,Gtk::FILL,Gtk::FILL,5);
  w_tag_table.attach(w_e_tag_genre,3,4,4,5,Gtk::FILL|Gtk::EXPAND,Gtk::FILL,5);
  
  w_e_tag_title.signal_key_release_event().connect(SigC::slot(*this,
                                           &FilePrefs::activate_tag_inserted));
  w_e_tag_artist.signal_key_release_event().connect(SigC::slot(*this,
                                            &FilePrefs::activate_tag_inserted));
  w_e_tag_album.signal_key_release_event().connect(SigC::slot(*this,
                                           &FilePrefs::activate_tag_inserted));
  w_e_tag_track.signal_key_release_event().connect(SigC::slot(*this,
                                           &FilePrefs::activate_tag_inserted));
  
  w_name_table.attach(w_e_prop_filename,0,1,0,1,Gtk::FILL|Gtk::EXPAND,
                                                Gtk::FILL,5);
  
  w_file_frame.add(w_file_table);	
  w_tag_frame.add(w_tag_table);
  w_name_frame.add(w_name_table);
  
  w_scrollwin_table.set_border_width(5);
  w_scrollwin_table.attach(w_file_frame,0,1,0,1,
                           Gtk::EXPAND|Gtk::FILL,Gtk::FILL);
  w_scrollwin_table.attach(w_tag_frame,0,1,1,2,
                           Gtk::EXPAND|Gtk::FILL,Gtk::FILL);
  w_scrollwin_table.attach(w_name_frame,0,1,2,3,
                           Gtk::EXPAND|Gtk::FILL,Gtk::FILL);
  
  w_global_scrollwin.set_shadow_type(Gtk::SHADOW_NONE);
  w_global_scrollwin.set_policy(Gtk::POLICY_AUTOMATIC,Gtk::POLICY_AUTOMATIC);
  w_global_scrollwin.add(w_scrollwin_table);
  dynamic_cast<Gtk::Viewport&>(*(w_global_scrollwin.get_child())).
                              set_shadow_type(Gtk::SHADOW_NONE);
  set_border_width(5);
  add(w_global_scrollwin);
  show_all();
}

FilePrefs::~FilePrefs()
{
}

Glib::ustring FilePrefs::get_e_prop_filename_text() const
{
  return w_e_prop_filename.get_text();
}
Glib::ustring FilePrefs::get_e_tag_title_text() const
{
  return w_e_tag_title.get_text();
}
Glib::ustring FilePrefs::get_e_tag_artist_text() const
{
  return w_e_tag_artist.get_text();
}
Glib::ustring FilePrefs::get_e_tag_album_text() const
{
  return w_e_tag_album.get_text();
}
unsigned char FilePrefs::get_e_tag_track_text() const
{
  return convstring(Glib::locale_from_utf8(w_e_tag_track.get_text()));
}
std::string FilePrefs::get_e_tag_genre_text() const
{
  return w_e_tag_genre.get_entry()->get_text();
}


void FilePrefs::set_e_filename_text(const Glib::ustring& str)
{
  w_e_filename.set_text(str);
}
void FilePrefs::set_e_filesize_text(const Glib::ustring& str)
{
  w_e_filesize.set_text(str);
}
void FilePrefs::set_e_tag_version_text(const Glib::ustring& str)
{
  w_e_tag_version.set_text(str);
}
void FilePrefs::set_e_tag_title_text(const Glib::ustring& str)
{
  w_e_tag_title.set_text(str);
}
void FilePrefs::set_e_tag_artist_text(const Glib::ustring& str)
{
  w_e_tag_artist.set_text(str);
}
void FilePrefs::set_e_tag_album_text(const Glib::ustring& str)
{
  w_e_tag_album.set_text(str);
}
void FilePrefs::set_e_tag_track_text(const Glib::ustring& str)
{
  w_e_tag_track.set_text(str);
}
void FilePrefs::set_e_tag_genre_text(const std::string& str)
{
  w_e_tag_genre.get_entry()->set_text(str);
}


void FilePrefs::set_e_prop_filename_text(const Glib::ustring& str)
{
  // replace "/" with "|"
  uint pos;
  Glib::ustring s(str);
  while( (pos = s.find("/")) != Glib::ustring::npos)
    s.replace(pos,1,"|");
  w_e_prop_filename.set_text(s);
  if(w_e_filename.get_text() == str)
    w_e_prop_filename.modify_text(Gtk::STATE_NORMAL,
                                  Gdk::Color("midnight blue"));
  else
    w_e_prop_filename.modify_text(Gtk::STATE_NORMAL,Gdk::Color(0));
}

void FilePrefs::reset_all_entries()
{
  w_e_filename.set_text("");
  w_e_filesize.set_text("");
  w_e_tag_version.set_text("");
  w_e_tag_title.set_text("");
  w_e_tag_artist.set_text("");
  w_e_tag_album.set_text("");
  w_e_tag_track.set_text("");
  w_e_prop_filename.set_text("");
  w_e_tag_genre.get_entry()->set_text("Unknown");
}

void FilePrefs::activate_rename_clicked()
{
  signal_rename_clicked();
}
bool FilePrefs::activate_tag_inserted(GdkEventKey* event)
{
  return signal_tag_inserted(event);
}


void FilePrefs::connect_rename_clicked(const SigC::Slot0<void>& slot_activated)
{
  signal_rename_clicked.connect(slot_activated);
}
void FilePrefs::connect_tag_inserted(const SigC::Slot1<bool,
                                     GdkEventKey*>& slot_activated)
{
  signal_tag_inserted.connect(slot_activated);
}

void FilePrefs::set_e_tags_sensitive(bool sens)
{
  w_tag_frame.set_sensitive(sens);
}
