/***************************************************************************
                        infodialog.cc  -  description
                             -------------------
    project              : GTagger
    version              : 0.1
    begin                : Sun Jan 10 2002
    lastchange           : Sun Jan 27 2002
    copyright            : (C) 2003 by Marcel Karras
    email                : toka@freebits.de
    homepage             : http://www.freebits.de
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#define  _(x) gettext(x)
#include <libintl.h>
#include "infodialog.h"
#include <gtkmm/box.h>
#include <gtkmm/alignment.h>
#include <gtkmm/label.h>
#include <gtkmm/stock.h>
#include <gtkmm/box.h>

InfoDialog::InfoDialog()
: gtagger_table(2,1),
  thankyou_table(1,1),
  about_table(1,1),
  info_table(2,3),
  a_info_img(Gtk::ALIGN_RIGHT, Gtk::ALIGN_CENTER, 0.0, 0.0),
  a_info_text(Gtk::ALIGN_LEFT, Gtk::ALIGN_CENTER, 0.0, 0.0),
  i_info(Gtk::Stock::DIALOG_INFO,Gtk::ICON_SIZE_DIALOG),
  a_info_author(Gtk::ALIGN_RIGHT, Gtk::ALIGN_CENTER, 0.0, 0.0),
  l_author(Glib::locale_to_utf8(_("Author:"))),
  a_info_author_text(Gtk::ALIGN_LEFT, Gtk::ALIGN_CENTER, 0.0, 0.0),
  l_author_text(Glib::locale_to_utf8(" Marcel Karras")),
  a_info_homepage(Gtk::ALIGN_RIGHT, Gtk::ALIGN_CENTER, 0.0, 0.0),
  l_homepage(Glib::locale_to_utf8(_("Homepage:"))),
  a_info_homepage_text(Gtk::ALIGN_LEFT, Gtk::ALIGN_CENTER, 0.0, 0.0),
  l_homepage_text(Glib::locale_to_utf8(" http://www.freebits.de")),  
  a_info_email(Gtk::ALIGN_RIGHT, Gtk::ALIGN_CENTER, 0.0, 0.0),
  l_email(Glib::locale_to_utf8(_("Email:"))),
  a_info_email_text(Gtk::ALIGN_LEFT, Gtk::ALIGN_CENTER, 0.0, 0.0),
  l_email_text(Glib::locale_to_utf8(" toka@freebits.de"))
{
  Gtk::VBox* box = get_vbox();
  
  a_info_author.add(l_author);
  a_info_author_text.add(l_author_text);
  a_info_homepage.add(l_homepage);
  a_info_homepage_text.add(l_homepage_text);
  a_info_email.add(l_email);
  a_info_email_text.add(l_email_text);
  info_table.attach(a_info_author,0,1,0,1,Gtk::FILL,Gtk::FILL,3);
  info_table.attach(a_info_author_text,1,2,0,1,Gtk::FILL,Gtk::FILL,3);
  info_table.attach(a_info_homepage,0,1,1,2,Gtk::FILL,Gtk::FILL,3);
  info_table.attach(a_info_homepage_text,1,2,1,2,Gtk::FILL,Gtk::FILL,3);
  info_table.attach(a_info_email,0,1,2,3,Gtk::FILL,Gtk::FILL,3);
  info_table.attach(a_info_email_text,1,2,2,3,Gtk::FILL,Gtk::FILL,3);
	
  a_info_text.add(info_table);
  a_info_img.add(i_info);
  Glib::RefPtr<Gtk::TextBuffer> textbuffer = Gtk::TextBuffer::create();
  textbuffer->set_text(Glib::locale_to_utf8(
      _("GTagger is an ID3Tag editor designed for easy and efficient usage. "
        "In order to fulfil the needs in manipulating tag information GTagger "
        "offers many useful features."
        "This first version (0.1) provides the following functions:"))+
      Glib::locale_to_utf8("\n\n")+Glib::locale_to_utf8(     
      _("- directory listing\n"
        "- viewing & editing ID3Tags\n"
        "- converting ID3Tags into several versions\n"
        "- filename suggestion suitable to tag information\n"
        "- live filename suggestion alteration while editing the ID3Tag\n"
        "- rename & delete files\n"
        "- short cuts for every function\n\n"
        "For more information have a look at:"))+
      Glib::locale_to_utf8(" http://www.freebits.de\n\n")+
      Glib::locale_to_utf8(_("If you encounter a bug please let me know:"))+
      Glib::locale_to_utf8(" toka@freebits.de"));
  w_textview.set_buffer(textbuffer);
  w_textview.set_editable(false);
  w_textview.set_cursor_visible(false);
  w_textview.set_wrap_mode(Gtk::WRAP_WORD);
  w_textview.set_left_margin(3);
  w_textview.set_right_margin(3);
  
  w_scrollwin.add(w_textview);
  w_scrollwin.set_policy(Gtk::POLICY_AUTOMATIC,Gtk::POLICY_AUTOMATIC);
  w_scrollwin.set_shadow_type(Gtk::SHADOW_NONE);
  
  gtagger_table.attach(a_info_img,0,1,0,1,Gtk::FILL|Gtk::EXPAND,
                                          Gtk::FILL|Gtk::EXPAND,5);
  gtagger_table.attach(a_info_text,1,2,0,1,Gtk::FILL|Gtk::EXPAND,
                                           Gtk::FILL|Gtk::EXPAND,5);
  thankyou_table.attach(w_acknowledgment,0,1,0,1,Gtk::FILL|Gtk::EXPAND,
                                                 Gtk::FILL|Gtk::EXPAND,5);
  about_table.attach(w_scrollwin,0,1,0,1,Gtk::FILL|Gtk::EXPAND, 
                                        Gtk::FILL|Gtk::EXPAND,10,10);
  
  notebook.append_page(gtagger_table,Glib::locale_to_utf8(
                                     _("GTagger")));
  notebook.append_page(thankyou_table,Glib::locale_to_utf8(
                                      _("Acknowledgment")));	
  notebook.append_page(about_table,Glib::locale_to_utf8(
                                   _("About")));	
  
  box->pack_start(notebook);
  add_button(Gtk::Stock::CLOSE, Gtk::RESPONSE_CLOSE);
  
  set_title(Glib::locale_to_utf8("GTagger"));
  set_default_size(500,370);
  set_has_separator(false);
  show_all();
}
	
InfoDialog::~InfoDialog()
{
}
