/***************************************************************************
                         menubar.cc  -  description
                             -------------------
    project              : GTagger
    version              : 0.1
    begin                : Sun Jan 10 2002
    lastchange           : Sun Jan 27 2002
    copyright            : (C) 2003 by Marcel Karras
    email                : toka@freebits.de
    homepage             : http://www.freebits.de
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#define  _(x) gettext(x)
#include <libintl.h>
#include "menubar.h"
#include "infodialog.h"
#include "pixmaps.h"
#include <gtkmm/main.h>
#include <gtkmm/stock.h>
#include <gdkmm/pixbuf.h>
#include <gtkmm/radiomenuitem.h>
#include <iostream>

MenuBar::MenuBar()
: w_rename_image(Gdk::Pixbuf::create_from_inline(sizeof(rename_file_small),
                                                 rename_file_small)),
  w_info_image(Gtk::Stock::DIALOG_INFO,Gtk::ICON_SIZE_MENU),
  w_tag_add_image(Gdk::Pixbuf::create_from_inline(sizeof(add_id3_tag_small),
                                                  add_id3_tag_small)),
  w_tag_del_image(Gdk::Pixbuf::create_from_inline(sizeof(del_id3_tag_small),
                                                  del_id3_tag_small)),
  w_tag_save_image(Gdk::Pixbuf::create_from_inline(sizeof(save_id3_tag_small),
                                                  save_id3_tag_small)),
  w_delete_image(Gdk::Pixbuf::create_from_inline(sizeof(delete_file_small),
                                                 delete_file_small)),
  w_reload_image(Gdk::Pixbuf::create_from_inline(sizeof(reload_list_small),
                                                 reload_list_small)),
  w_save_all_image(Gdk::Pixbuf::create_from_inline(sizeof(save_all_small),
                                                   save_all_small))
{
  items().push_back( Gtk::Menu_Helpers::MenuElem(
	             Glib::locale_to_utf8(_("_File")), w_menu_file) );
  items().push_back( Gtk::Menu_Helpers::MenuElem(
	             Glib::locale_to_utf8(_("_Edit")), w_menu_edit) );
  items().push_back( Gtk::Menu_Helpers::MenuElem(
	             Glib::locale_to_utf8(_("_Help")), w_menu_help) );
  Gtk::Menu::MenuList& w_menulist_file = w_menu_file.items();
  Gtk::Menu::MenuList& w_menulist_edit = w_menu_edit.items();
  Gtk::Menu::MenuList& w_menulist_help = w_menu_help.items();
  Gtk::RadioMenuItem_Helpers::Group radio_group;
  w_menulist_file.push_back( Gtk::Menu_Helpers::StockMenuElem(
    Gtk::Stock::OPEN,SigC::slot(*this, &MenuBar::activate_file_open)));
  w_menulist_file.push_back( Gtk::Menu_Helpers::StockMenuElem(
    Gtk::Stock::QUIT,SigC::slot(&Gtk::Main::quit)));
  w_menulist_edit.push_back( Gtk::Menu_Helpers::RadioMenuElem(
    radio_group, Glib::locale_to_utf8(_("ID3v_1")),
    Gtk::Menu_Helpers::AccelKey("<control>1"),
    SigC::slot(*this, &MenuBar::activate_tag_version_changed)));
  w_menulist_edit.push_back( Gtk::Menu_Helpers::RadioMenuElem(
    radio_group, Glib::locale_to_utf8(_("ID3v_2")),
    Gtk::Menu_Helpers::AccelKey("<control>2"),
    SigC::slot(*this, &MenuBar::activate_tag_version_changed)));
  w_menulist_edit.push_back( Gtk::Menu_Helpers::RadioMenuElem(
    radio_group, Glib::locale_to_utf8(_("ID3v1 _and ID3v2")),
    Gtk::Menu_Helpers::AccelKey("<control>3"),
    SigC::slot(*this, &MenuBar::activate_tag_version_changed)));
  w_menulist_edit.push_back( Gtk::Menu_Helpers::SeparatorElem());
  w_menulist_edit.push_back( Gtk::Menu_Helpers::ImageMenuElem(
    Glib::locale_to_utf8(_("_Create ID3Tag")), 
    Gtk::Menu_Helpers::AccelKey("<control>Insert"), w_tag_add_image,
    SigC::slot(*this, &MenuBar::activate_tag_add)));
  w_menulist_edit.push_back( Gtk::Menu_Helpers::ImageMenuElem(
    Glib::locale_to_utf8(_("_Delete ID3Tag")), 
    Gtk::Menu_Helpers::AccelKey("<control>Delete"), w_tag_del_image,
    SigC::slot(*this, &MenuBar::activate_tag_del)));
  w_menulist_edit.push_back( Gtk::Menu_Helpers::SeparatorElem());
  w_menulist_edit.push_back( Gtk::Menu_Helpers::ImageMenuElem(
    Glib::locale_to_utf8(_("_Save/Convert ID3Tag")), 
    Gtk::Menu_Helpers::AccelKey("<control>s"), w_tag_save_image,
    SigC::slot(*this, &MenuBar::activate_tag_save)));
  w_menulist_edit.push_back( Gtk::Menu_Helpers::ImageMenuElem(
    Glib::locale_to_utf8(_("_Rename File")), 
    Gtk::Menu_Helpers::AccelKey("<control>r"), w_rename_image,
    SigC::slot(*this, &MenuBar::activate_file_rename)));
  w_menulist_edit.push_back( Gtk::Menu_Helpers::ImageMenuElem(
    Glib::locale_to_utf8(_("Sa_ve/Convert ID3Tag & Rename File")), 
    Gtk::Menu_Helpers::AccelKey("<control>w"), w_save_all_image,
    SigC::slot(*this, &MenuBar::activate_save_all_pressed)));
  w_menulist_edit.push_back( Gtk::Menu_Helpers::SeparatorElem());
  w_menulist_edit.push_back( Gtk::Menu_Helpers::ImageMenuElem(
    Glib::locale_to_utf8(_("Delete _File")), 
    Gtk::Menu_Helpers::AccelKey("<control>d"), w_delete_image,
    SigC::slot(*this, &MenuBar::activate_file_delete)));
  w_menulist_edit.push_back( Gtk::Menu_Helpers::SeparatorElem());
  w_menulist_edit.push_back( Gtk::Menu_Helpers::ImageMenuElem(
    Glib::locale_to_utf8(_("Reload _List")), 
    Gtk::Menu_Helpers::AccelKey("<control>a"), w_reload_image,
    SigC::slot(*this, &MenuBar::activate_reload_pressed)));
  w_menulist_help.push_back( Gtk::Menu_Helpers::ImageMenuElem(
    Glib::locale_to_utf8(_("_Info")),
    Gtk::Menu_Helpers::AccelKey("<control>i"),w_info_image, 
    SigC::slot(*this,&MenuBar::on_help_info_clicked)));
  items().back().set_right_justified();
  set_version_button(2);
  show_all();
}
	
MenuBar::~MenuBar()
{
}

void MenuBar::on_help_info_clicked()
{
  InfoDialog d_info;
  d_info.run();
}

void MenuBar::activate_file_open()
{
  signal_file_open();
}
void MenuBar::activate_file_rename()
{
  signal_file_rename();
}
void MenuBar::activate_file_delete()
{
  signal_file_delete();
}
void MenuBar::activate_tag_add()
{
  signal_tag_add();
}
void MenuBar::activate_tag_del()
{
  signal_tag_del();
}
void MenuBar::activate_tag_save()
{
  signal_tag_save();
}
void MenuBar::activate_tag_version_changed()
{
  signal_tag_version_changed(0);
}
void MenuBar::activate_reload_pressed()
{
  signal_reload_pressed();
}
void MenuBar::activate_save_all_pressed()
{
  signal_save_all_pressed();
}


void MenuBar::connect_file_open(const SigC::Slot0<void>& slot_activated)
{
  signal_file_open.connect(slot_activated);
}
void MenuBar::connect_file_rename(const SigC::Slot0<void>& slot_activated)
{
  signal_file_rename.connect(slot_activated);
}
void MenuBar::connect_file_delete(const SigC::Slot0<void>& slot_activated)
{
  signal_file_delete.connect(slot_activated);
}
void MenuBar::connect_tag_add(const SigC::Slot0<void>& slot_activated)
{
  signal_tag_add.connect(slot_activated);
}
void MenuBar::connect_tag_del(const SigC::Slot0<void>& slot_activated)
{
  signal_tag_del.connect(slot_activated);
}
void MenuBar::connect_tag_save(const SigC::Slot0<void>& slot_activated)
{
  signal_tag_save.connect(slot_activated);
}
void MenuBar::connect_tag_version_changed(const SigC::Slot1<void,
                                          int>& slot_activated)
{
  signal_tag_version_changed.connect(slot_activated);
}
void MenuBar::connect_reload_pressed(const SigC::Slot0<void>& slot_activated)
{
  signal_reload_pressed.connect(slot_activated);
}
void MenuBar::connect_save_all_pressed(const SigC::Slot0<void>& slot_activated)
{
  signal_save_all_pressed.connect(slot_activated);
}



void MenuBar::set_rename_sensitive(bool sens)
{
  Gtk::Menu::MenuList& w_menulist_edit = w_menu_edit.items();
  (w_menulist_edit[8]).set_sensitive(sens);
}
void MenuBar::set_delete_sensitive(bool sens)
{
  Gtk::Menu::MenuList& w_menulist_edit = w_menu_edit.items();
  (w_menulist_edit[11]).set_sensitive(sens);
}
void MenuBar::set_tag_save_sensitive(bool sens)
{
  Gtk::Menu::MenuList& w_menulist_edit = w_menu_edit.items();
  (w_menulist_edit[7]).set_sensitive(sens);
}
void MenuBar::set_tag_del_sensitive(bool sens)
{
  Gtk::Menu::MenuList& w_menulist_edit = w_menu_edit.items();
  (w_menulist_edit[5]).set_sensitive(sens);
}
void MenuBar::set_tag_add_sensitive(bool sens)
{
  Gtk::Menu::MenuList& w_menulist_edit = w_menu_edit.items();
  (w_menulist_edit[4]).set_sensitive(sens);
}
void MenuBar::set_save_all_sensitive(bool sens)
{
  Gtk::Menu::MenuList& w_menulist_edit = w_menu_edit.items();
  (w_menulist_edit[9]).set_sensitive(sens);
}


void MenuBar::set_version_button(int version)
{
  Gtk::RadioMenuItem* t1(dynamic_cast<Gtk::RadioMenuItem*>(
	                 &((w_menu_edit.items())[0])));
  Gtk::RadioMenuItem* t2(dynamic_cast<Gtk::RadioMenuItem*>(
                         &((w_menu_edit.items())[1])));
  Gtk::RadioMenuItem* t3(dynamic_cast<Gtk::RadioMenuItem*>(
                         &((w_menu_edit.items())[2])));
  switch(version)
  {
    case 1: t1->set_active(true);break;
    case 2: t2->set_active(true);break;
    case 3: t3->set_active(true);break;
    default: break;
  }
}

int MenuBar::get_version_button() const
{
  Gtk::RadioMenuItem* t1(dynamic_cast<Gtk::RadioMenuItem*>(
                         &((w_menu_edit.items())[0])));
  Gtk::RadioMenuItem* t2(dynamic_cast<Gtk::RadioMenuItem*>(
                         &((w_menu_edit.items())[1])));
  Gtk::RadioMenuItem* t3(dynamic_cast<Gtk::RadioMenuItem*>(
                         &((w_menu_edit.items())[2])));
  if(t1->get_active())
    return 1;
  if(t2->get_active())
    return 2;
  if(t3->get_active())
    return 3;
  return 0;	
}
