/***************************************************************************
                         status.cc  -  description
                             -------------------
    project              : GTagger
    version              : 0.1
    begin                : Sun Jan 20 2002
    lastchange           : Sun Jan 27 2002
    copyright            : (C) 2003 by Marcel Karras
    email                : toka@freebits.de
    homepage             : http://www.freebits.de
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#define  _(x) gettext(x)
#include <libintl.h>
#include "status.h"
#include <gtkmm/main.h>
#include <iostream>

Status::Status()
: Table(2,1)
{
  w_left_bar.set_has_resize_grip(false);
  w_right_bar.set_size_request(250);
  attach(w_left_bar,0,1,0,1,Gtk::FILL|Gtk::EXPAND,Gtk::FILL,0);
  attach(w_right_bar,1,2,0,1,Gtk::FILL,Gtk::FILL,0);
  show_all();
}
	
Status::~Status()
{
}

void Status::set_text(const Glib::ustring& s, int statusbar, int timeout)
{
  switch(statusbar)
  {
    // left
    case 0:
    {	
      if(!timeout)            // timeout is not set -> undeterminated
      {
        w_left_bar.pop();
        w_left_bar.push(s);
      }
      else                    // set text and remove it after given time
      {
        // remove text and timeouts
        remove_text(statusbar);
        w_left_bar.push(s); 
        // set new timeout
        SigC::Slot0<bool> my_slot = 
          SigC::bind(SigC::slot(*this,&Status::timer_callback),statusbar);
        conn_left =  Glib::signal_timeout().connect(my_slot,timeout);	
        break;
      }
    }
    // right
    case 1: 
    {
      if(!timeout)            // timeout is not set -> undeterminated
      {
        w_right_bar.pop(); 
        w_right_bar.push(s);
      }
      else                    // set text and remove it after given time
      {
        // remove text and timeouts
        remove_text(statusbar);
        w_right_bar.push(s); 
        // set new timeout
        SigC::Slot0<bool> my_slot = 
          SigC::bind(SigC::slot(*this,&Status::timer_callback),statusbar);
        conn_right =  Glib::signal_timeout().connect(my_slot,timeout);	
        break;
      }
    }
    default: break;
  }
}

void Status::remove_text(int statusbar)
{
  switch(statusbar)
  {
    // left
    case 0: w_left_bar.pop(); timer_disconnect(statusbar); break;
    // right
    case 1: w_right_bar.pop(); timer_disconnect(statusbar); break;
    default: break;
  }
}

bool Status::has_timeout(int statusbar)
{
  switch(statusbar)
  {
    // left
    case 0: return conn_left.connected();
    // right
    case 1: return conn_right.connected();
    default: return false;
  }
}

bool Status::timer_callback(int statusbar)
{
  remove_text(statusbar);
  return false;	// terminate timeout
}

void Status::timer_disconnect(int statusbar)
{
  switch(statusbar)
  {
    case 0:
    {
      if(conn_left.connected())     // disconnect
        conn_left.disconnect();
      break;      
    }
    case 1:
    {
      if(conn_right.connected())    // disconnect
        conn_right.disconnect();
      break;      
    }
  }
}
