/***************************************************************************
                        toolbar.cc  -  description
                             -------------------
    project              : GTagger
    version              : 0.1
    begin                : Sun Jan 14 2002
    lastchange           : Sun Jan 27 2002
    copyright            : (C) 2003 by Marcel Karras
    email                : toka@freebits.de
    homepage             : http://www.freebits.de
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#define  _(x) gettext(x)
#include <libintl.h>
#include "toolbar.h"
#include "pixmaps.h"
#include <gdkmm/pixbuf.h>
#include <gtkmm/main.h>
#include <gtkmm/stock.h>
#include <gtkmm/widget.h>

ToolBar::ToolBar()
: b_delete_file_image(Gdk::Pixbuf::create_from_inline(sizeof(delete_file),
                                                      delete_file)),
  b_rename_file_image(Gdk::Pixbuf::create_from_inline(sizeof(rename_file),
                                                      rename_file)),
  b_tag_file_image(Gdk::Pixbuf::create_from_inline(sizeof(save_id3_tag),
                                                   save_id3_tag)),
  b_save_all_image(Gdk::Pixbuf::create_from_inline(sizeof(save_all),
                                                   save_all)),
  b_tag_add_file_image(Gdk::Pixbuf::create_from_inline(sizeof(add_id3_tag),
                                                       add_id3_tag)),
  b_tag_del_file_image(Gdk::Pixbuf::create_from_inline(sizeof(del_id3_tag),
                                                       del_id3_tag)),
  b_id3v1_image(Gdk::Pixbuf::create_from_inline(sizeof(id3_v1),
                                                id3_v1)),
  b_id3v2_image(Gdk::Pixbuf::create_from_inline(sizeof(id3_v2),
                                                id3_v2)),
  b_id3v12_image(Gdk::Pixbuf::create_from_inline(sizeof(id3_v12),
                                                 id3_v12)),
  b_reload_image(Gdk::Pixbuf::create_from_inline(sizeof(reload_list),
                                                 reload_list))
{
  b_delete_file.set_relief(Gtk::RELIEF_NONE);	
  b_delete_file.add(b_delete_file_image);
  b_rename_file.set_relief(Gtk::RELIEF_NONE);	
  b_rename_file.add(b_rename_file_image);
  b_save_all.set_relief(Gtk::RELIEF_NONE);	
  b_save_all.add(b_save_all_image);
  b_tag_file.set_relief(Gtk::RELIEF_NONE);	
  b_tag_file.add(b_tag_file_image);
  b_tag_add_file.set_relief(Gtk::RELIEF_NONE);	
  b_tag_add_file.add(b_tag_add_file_image);
  b_tag_del_file.set_relief(Gtk::RELIEF_NONE);	
  b_tag_del_file.add(b_tag_del_file_image);
  b_reload_list.set_relief(Gtk::RELIEF_NONE);	
  b_reload_list.add(b_reload_image);
  b_delete_file.signal_clicked().connect(SigC::slot(*this,
                                         &ToolBar::activate_delete_pressed));
  b_rename_file.signal_clicked().connect(SigC::slot(*this,
                                         &ToolBar::activate_rename_pressed));
  b_tag_file.signal_clicked().connect(SigC::slot(*this,
                                      &ToolBar::activate_tag_pressed));
  b_tag_add_file.signal_clicked().connect(SigC::slot(*this,
                                          &ToolBar::activate_tag_add_pressed));
  b_tag_del_file.signal_clicked().connect(SigC::slot(*this,
                                          &ToolBar::activate_tag_del_pressed));
  b_reload_list.signal_clicked().connect(SigC::slot(*this,
                                         &ToolBar::activate_reload_pressed));
  b_save_all.signal_clicked().connect(SigC::slot(*this,
                                      &ToolBar::activate_save_all_pressed));
  
  Gtk::Toolbar::ToolList& tools(tools());
  tools.push_back(Gtk::Toolbar_Helpers::StockElem(Gtk::Stock::QUIT, 
                  Gtk::Main::quit, 
                  Glib::locale_to_utf8(_("Quit program (Ctrl+Q)"))));
  tools.push_back(Gtk::Toolbar_Helpers::StockElem(Gtk::Stock::OPEN, 
                  SigC::slot(*this,&ToolBar::activate_open_pressed), 
                  Glib::locale_to_utf8(_("Open directory (Ctrl+O)"))));
  tools.push_back(Gtk::Toolbar_Helpers::Space());
  tools.push_back(Gtk::Toolbar_Helpers::Element(b_tag_add_file, 
                  Glib::locale_to_utf8(_("Create ID3Tag (Ctrl+Ins)"))));
  tools.push_back(Gtk::Toolbar_Helpers::Element(b_tag_del_file, 
                  Glib::locale_to_utf8(_("Delete ID3Tag (Ctrl+Del)"))));
  tools.push_back(Gtk::Toolbar_Helpers::Space());
  tools.push_back(Gtk::Toolbar_Helpers::Element(b_tag_file, 
                  Glib::locale_to_utf8(_("Save/Convert ID3Tag (Ctrl+S)"))));
  tools.push_back(Gtk::Toolbar_Helpers::Element(b_rename_file, 
                  Glib::locale_to_utf8(_("Rename File (Ctrl+R)"))));
  tools.push_back(Gtk::Toolbar_Helpers::Element(b_save_all, 
                  Glib::locale_to_utf8(
                  _("Save/Convert ID3Tag & Rename File (Ctrl+W)"))));
  tools.push_back(Gtk::Toolbar_Helpers::Space());
  tools.push_back(Gtk::Toolbar_Helpers::Element(b_delete_file, 
                  Glib::locale_to_utf8(_("Delete File (Ctrl+D)"))));
  tools.push_back(Gtk::Toolbar_Helpers::Space());
	
  Gtk::RadioButton_Helpers::Group	group;
  
  tools.push_back(Gtk::Toolbar_Helpers::RadioElem(group, "id3v1", 
                  b_id3v1_image, 
                  SigC::slot(*this,&ToolBar::activate_tag_version_changed), 
                  Glib::locale_to_utf8(
                  _("Save tags in ID3v1 format (Ctrl+1)"))));
  p_id3v1 = dynamic_cast<Gtk::RadioButton*>(tools.back().get_widget());
  tools.push_back(Gtk::Toolbar_Helpers::RadioElem(group, "id3v2", 
                  b_id3v2_image, 
                  SigC::slot(*this,&ToolBar::activate_tag_version_changed), 
                  Glib::locale_to_utf8(
                  _("Save tags in ID3v2 format (Ctrl+2)"))));
  p_id3v2 = dynamic_cast<Gtk::RadioButton*>(tools.back().get_widget());
  tools.push_back(Gtk::Toolbar_Helpers::RadioElem(group, "id3v12", 
                  b_id3v12_image, 
                  SigC::slot(*this,&ToolBar::activate_tag_version_changed), 
                  Glib::locale_to_utf8(
                  _("Save tags in ID3 v1&v2 format (Ctrl+3)"))));
  p_id3v12 = dynamic_cast<Gtk::RadioButton*>(tools.back().get_widget());
  tools.push_back(Gtk::Toolbar_Helpers::Space());
  tools.push_back(Gtk::Toolbar_Helpers::Element(b_reload_list, 
                  Glib::locale_to_utf8(_("Reload List (Ctrl+A)"))));
  set_toolbar_style(Gtk::TOOLBAR_ICONS);
  set_version_button(2);
  show_all();
}
	
ToolBar::~ToolBar()
{
}

void ToolBar::b_delete_file_sensitive(bool sensitive)
{
  b_delete_file.set_sensitive(sensitive);
}
void ToolBar::b_rename_file_sensitive(bool sensitive)
{
  b_rename_file.set_sensitive(sensitive);
}
void ToolBar::b_tag_file_sensitive(bool sensitive)
{
  b_tag_file.set_sensitive(sensitive);
}
void ToolBar::b_tag_add_file_sensitive(bool sensitive)
{
  b_tag_add_file.set_sensitive(sensitive);
}
void ToolBar::b_tag_del_file_sensitive(bool sensitive)
{
  b_tag_del_file.set_sensitive(sensitive);
}
void ToolBar::b_save_all_sensitive(bool sensitive)
{
  b_save_all.set_sensitive(sensitive);
}

void ToolBar::activate_delete_pressed()
{
  signal_delete_pressed();
}
void ToolBar::activate_open_pressed()
{
  signal_open_pressed();
}
void ToolBar::activate_rename_pressed()
{
  signal_rename_pressed();
}
void ToolBar::activate_tag_pressed()
{
  signal_tag_pressed();
}
void ToolBar::activate_tag_add_pressed()
{
  signal_tag_add_pressed();
}
void ToolBar::activate_tag_del_pressed()
{
  signal_tag_del_pressed();
}
void ToolBar::activate_tag_version_changed()
{
  signal_tag_version_changed(1);
}
void ToolBar::activate_reload_pressed()
{
  signal_reload_pressed();
}
void ToolBar::activate_save_all_pressed()
{
  signal_save_all_pressed();
}


void ToolBar::connect_delete_pressed(const SigC::Slot0<void>& slot_activated)
{
  signal_delete_pressed.connect(slot_activated);
}
void ToolBar::connect_open_pressed(const SigC::Slot0<void>& slot_activated)
{
  signal_open_pressed.connect(slot_activated);
}
void ToolBar::connect_rename_pressed(const SigC::Slot0<void>& slot_activated)
{
  signal_rename_pressed.connect(slot_activated);
}
void ToolBar::connect_tag_pressed(const SigC::Slot0<void>& slot_activated)
{
  signal_tag_pressed.connect(slot_activated);
}
void ToolBar::connect_tag_add_pressed(const SigC::Slot0<void>& slot_activated)
{
  signal_tag_add_pressed.connect(slot_activated);
}
void ToolBar::connect_tag_del_pressed(const SigC::Slot0<void>& slot_activated)
{
  signal_tag_del_pressed.connect(slot_activated);
}
void ToolBar::connect_tag_version_changed(const SigC::Slot1<void,int>& 
                                          slot_activated)
{
  signal_tag_version_changed.connect(slot_activated);
}
void ToolBar::connect_reload_pressed(const SigC::Slot0<void>& slot_activated)
{
  signal_reload_pressed.connect(slot_activated);
}
void ToolBar::connect_save_all_pressed(const SigC::Slot0<void>& slot_activated)
{
  signal_save_all_pressed.connect(slot_activated);
}


void ToolBar::set_version_button(int version)
{
  switch(version)
  {
    case 1: p_id3v1->set_active(true);break;
    case 2: p_id3v2->set_active(true);break;
    case 3: p_id3v12->set_active(true);break;
    default: break;
  }
}

int ToolBar::get_version_button() const
{
  if(p_id3v1->get_active())
    return 1;
  if(p_id3v2->get_active())
    return 2;
  if(p_id3v12->get_active())
    return 3;
  return 0;
}
