/***************************************************************************
                      windowtoplevel.cc  -  description
                             -------------------
    project              : GTagger
    version              : 0.1
    begin                : Sun Jan 10 2002
    lastchange           : Sun Jan 27 2002
    copyright            : (C) 2003 by Marcel Karras
    email                : toka@freebits.de
    homepage             : http://www.freebits.de
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#define  _(x) gettext(x)
#include <libintl.h>
#include "windowtoplevel.h"
#include "errordialog.h"
#include "tagadder.h"
#include "pixmaps.h"
#include "genre.h"
#include <sstream>
#include <gtkmm/fileselection.h>
#include <gtkmm/messagedialog.h>
#include <gtkmm/treemodel.h>
#include <gdkmm/pixbuf.h>
#include <sys/stat.h>
#include <dirent.h>
#include <errno.h>
#include <ctype.h>
#include <string.h>
#include <unistd.h>
#include <time.h>
#include <iostream>
#include <iomanip>
#include <vector>
#include <algorithm>
#include <cstdio>
#include <id3/tag.h>
#include <id3/misc_support.h>
extern int errno;

WindowToplevel::WindowToplevel()
: w_toplevel_table(1,4),
  s_current_tag_version("id3v2"),
  all_files(0)
{
  // don't care if files aren't UTF8 compatible
  setenv("G_BROKEN_FILENAMES","1",1);
  // try to cover current working dir
  Glib::ustring current_working_dir(getenv("PWD"));
  if(current_working_dir.empty())
    current_working_dir = getenv("HOME");
  s_current_dir = current_working_dir+"/";

  w_toplevel_vpaned.set_position(300);
  w_toplevel_vpaned.add1(w_file_prefs);
  w_toplevel_vpaned.add2(w_file_browse);
	
  w_toplevel_table.attach(w_menu_bar,0,1,0,1,Gtk::FILL,Gtk::FILL);
  w_toplevel_table.attach(w_tool_bar,0,1,1,2,Gtk::FILL,Gtk::FILL);
  w_toplevel_table.attach(w_toplevel_vpaned,0,1,2,3);
  w_toplevel_table.attach(w_statusbar,0,1,3,4,Gtk::FILL,Gtk::FILL);


  add(w_toplevel_table);
  set_default_size(600,600);
  set_title(Glib::locale_to_utf8(_("GTagger - ID3 Tag Editor")));

  w_menu_bar.connect_file_open(SigC::slot(
             *this,&WindowToplevel::s_button_open_clicked));
  w_menu_bar.connect_file_rename(SigC::slot(
             *this,&WindowToplevel::s_button_rename_clicked));
  w_menu_bar.connect_file_delete(SigC::slot(
             *this,&WindowToplevel::s_button_delete_clicked));
  w_menu_bar.connect_tag_add(SigC::slot(
             *this,&WindowToplevel::s_button_tag_add_clicked));
  w_menu_bar.connect_tag_del(SigC::slot(
             *this,&WindowToplevel::s_button_tag_del_clicked));
  w_menu_bar.connect_tag_save(SigC::slot(
             *this,&WindowToplevel::s_button_tag_clicked));
  w_menu_bar.connect_tag_version_changed(SigC::slot(
             *this,&WindowToplevel::set_tag_version));
  w_menu_bar.connect_reload_pressed(SigC::slot(
             *this,&WindowToplevel::s_reload_pressed));
  w_menu_bar.connect_save_all_pressed(SigC::slot(
             *this,&WindowToplevel::s_save_all_pressed));
  w_file_browse.connect_row_changed(SigC::slot(
             *this,&WindowToplevel::reload_properties));
  w_file_prefs.connect_rename_clicked(SigC::slot(
             *this,&WindowToplevel::s_button_rename_clicked));
  w_file_prefs.connect_tag_inserted(SigC::slot(
             *this,&WindowToplevel::tag_changed));
  w_tool_bar.connect_delete_pressed(SigC::slot(
             *this,&WindowToplevel::s_button_delete_clicked));
  w_tool_bar.connect_open_pressed(SigC::slot(
             *this,&WindowToplevel::s_button_open_clicked));
  w_tool_bar.connect_rename_pressed(SigC::slot(
             *this,&WindowToplevel::s_button_rename_clicked));
  w_tool_bar.connect_tag_del_pressed(SigC::slot(
             *this,&WindowToplevel::s_button_tag_del_clicked));
  w_tool_bar.connect_tag_add_pressed(SigC::slot(
             *this,&WindowToplevel::s_button_tag_add_clicked));
  w_tool_bar.connect_tag_pressed(SigC::slot(
             *this,&WindowToplevel::s_button_tag_clicked));
  w_tool_bar.connect_tag_version_changed(SigC::slot(
             *this,&WindowToplevel::set_tag_version));
  w_tool_bar.connect_reload_pressed(SigC::slot(
             *this,&WindowToplevel::s_reload_pressed));
  w_tool_bar.connect_save_all_pressed(SigC::slot(
             *this,&WindowToplevel::s_save_all_pressed));
	     
  w_file_prefs.set_sensitive(false);
  w_tool_bar.b_delete_file_sensitive(false);
  w_tool_bar.b_rename_file_sensitive(false);
  w_tool_bar.b_tag_file_sensitive(false);
  w_tool_bar.b_tag_add_file_sensitive(false);
  w_tool_bar.b_tag_del_file_sensitive(false);
  w_tool_bar.b_save_all_sensitive(false);
  w_menu_bar.set_rename_sensitive(false);
  w_menu_bar.set_delete_sensitive(false);
  w_menu_bar.set_tag_save_sensitive(false);
  w_menu_bar.set_tag_add_sensitive(false);
  w_menu_bar.set_tag_del_sensitive(false);  
  w_menu_bar.set_save_all_sensitive(false);  

  std::list< Glib::RefPtr<Gdk::Pixbuf> > icons;
  icons.push_back(Gdk::Pixbuf::create_from_inline(sizeof(prog_icon),prog_icon));
  set_default_icon_list(icons);	
  show_all();
  s_button_open_clicked();
}
	
WindowToplevel::~WindowToplevel()
{
}

void WindowToplevel::s_button_open_clicked()
throw(FileSystemException)
{
  Gtk::FileSelection d_folderselection(
                     Glib::locale_to_utf8(_("Choose a directory...")));
  d_folderselection.set_filename(Glib::filename_from_utf8(s_current_dir));
  d_folderselection.hide_fileop_buttons();
  d_folderselection.get_file_list()->set_sensitive(false);
  d_folderselection.set_transient_for(*this);
  int result = d_folderselection.run();
  switch(result)
  {
    case(Gtk::RESPONSE_OK):
    {
      Glib::ustring s_old_dir = s_current_dir;
      s_current_dir = Glib::filename_to_utf8(d_folderselection.get_filename());
      struct stat attribut;
      try
      {
        if(lstat(Glib::filename_from_utf8(s_current_dir).c_str(), 
                 &attribut) == -1)
          throw(FileSystemException(FileSystemException::FS_STAT_DIR,
                                    Glib::locale_to_utf8(strerror(errno))));
        if (S_ISDIR(attribut.st_mode))
        {
          DIR *p_dir;
          if( (p_dir = opendir(
               Glib::filename_from_utf8(s_current_dir).c_str())) == NULL)
          {
            closedir(p_dir);
            throw(FileSystemException(FileSystemException::FS_STAT_DIR));
          }
          w_file_browse.clear();
          
	  struct dirent *p_dirent;
          unsigned long files=0;
          std::vector<Glib::ustring> entries;
          struct stat att;
          
          while( (p_dirent = readdir(p_dir)) != NULL)
          {
            if(strcmp(p_dirent->d_name,".") && strcmp(p_dirent->d_name, ".."))
            {
              Glib::ustring tmp = Glib::filename_to_utf8(p_dirent->d_name);
              if(lstat(Glib::filename_from_utf8(s_current_dir+tmp).c_str(), 
                       &att) != -1)
              {
                if (S_ISREG(att.st_mode))
                {
                  entries.push_back(Glib::filename_to_utf8(p_dirent->d_name));
                  files++;
                }
              }
            }
          }
          closedir(p_dir);
          std::stable_sort(entries.begin(),entries.end());
          for(uint i=0; i<entries.size(); i++)
          {
            if(entries[i].size() > 2)
            {
              Glib::ustring s(entries[i],entries[i].size()-3,3);
              if( (s == "mp3") || (s == "mP3") || (s == "Mp3") || (s == "MP3")
              ||  (s == "ogg") || (s == "Ogg") || (s == "OGg") || (s == "OgG")
              ||  (s == "oGG") || (s == "ogG") || (s == "oGg")) 
                w_file_browse.appendRow(entries[i], FileBrowse::SOUND_MIME);
              else
                w_file_browse.appendRow(entries[i], FileBrowse::UNKNOWN_MIME);
            }
            else
              w_file_browse.appendRow(entries[i], FileBrowse::UNKNOWN_MIME);								
          }
          set_files(files);
          no_selection();
        }
      }
      catch(const FileSystemException& exp)
      {
        s_current_dir = s_old_dir;
        exp.showErrorDialog();
      }
      catch(...)
      {
        s_current_dir = s_old_dir;
        ErrorDialog d_error(
                    Glib::locale_to_utf8(_("An unknown error has occured.")));
        d_error.run();
      }
      break;
    }
    default: break;
  }
}

void WindowToplevel::reload_properties()
{	
  s_last_file = w_file_browse.getCurrentRowContent();
  struct stat attribut;
  std::string s_unicode = 
		Glib::filename_from_utf8(s_current_dir+s_last_file);
  if(!s_last_file.empty())
  {
    Glib::ustring file_size;
    Glib::ustring title;
    Glib::ustring artist;
    Glib::ustring album;
    Glib::ustring track;
    Glib::ustring proprename;
    std::string   genre;
    // 0 = no Tag
    // 1 = ID3V1
    // 2 = ID3V2
    // 3 = ID3V1&2
    int hastag=0;
    try
    {
      if(lstat(s_unicode.c_str(), &attribut) == -1)
        throw(FileSystemException(FileSystemException::FS_STAT_FILE,
                                  Glib::locale_to_utf8(strerror(errno))));
      else
      {
        std::ostringstream os;
        os.setf(std::ios::fixed);
        os << uint(attribut.st_size) << " Bytes (" << std::setprecision(3) 
           << (double(attribut.st_size)/1024/1024) << " MB )";
        file_size = Glib::locale_to_utf8(os.str());
        // check for ID3Tag Versions
        ID3_Tag Tag;
        Tag.Clear();
        Tag.Link(s_unicode.c_str());
        if(Tag.HasV1Tag())
          hastag+=1;
        if(Tag.HasV2Tag())
          hastag+=2;				
        ID3_Frame* Frame = Tag.Find(ID3FID_TITLE);
        if (Frame != NULL)
        {
          ID3_Field* Field = Frame->GetField(ID3FN_TEXT);
          if(Field != NULL)
          {
            std::string str("");
            if(Field->GetRawText() != NULL)
              str = Field->GetRawText();
            if(!str.empty())
              title = Glib::locale_to_utf8(str);
          }
        }
        Frame = Tag.Find(ID3FID_LEADARTIST);
        if (Frame != NULL)
        {			
          ID3_Field* Field = Frame->GetField(ID3FN_TEXT);
          if(Field != NULL)
          {
            std::string str("");
            if(Field->GetRawText() != NULL)
              str = Field->GetRawText();
            if(!str.empty())
              artist = Glib::locale_to_utf8(str);
          }
        }
        Frame = Tag.Find(ID3FID_ALBUM);
        if (Frame != NULL)
        {
          ID3_Field* Field = Frame->GetField(ID3FN_TEXT);
          if(Field != NULL)
          {
            std::string str("");
            if(Field->GetRawText() != NULL)
              str = Field->GetRawText();
            if(!str.empty())
              album = Glib::locale_to_utf8(str);
          }
        }
        Frame = Tag.Find(ID3FID_TRACKNUM);
        if (Frame != NULL)
        {
          ID3_Field* Field = Frame->GetField(ID3FN_TEXT);
          if(Field != NULL)
          {
            std::string str("");
            if(Field->GetRawText() != NULL)
              str = Field->GetRawText();
            if(!str.empty())
              track = Glib::locale_to_utf8(str);
          }
        }
        Frame = Tag.Find(ID3FID_CONTENTTYPE);
        if (Frame != NULL)
        {	
          int tmp = 0;
          std::sscanf(ID3_GetString(Frame, ID3FN_TEXT), "(%d)", &tmp);
          if(tmp<148 && tmp>-1)
            genre = std::string(genre_table[tmp]);
        }				
        if(hastag)
        {
          Glib::ustring tr;
          if(track.size()>0){
            if(track.size()<2){
              tr = "0"+track;
            }
            else
              tr = track;
          }
          if (!title.empty() && !artist.empty() &&
              !album.empty() && !track.empty())
          {
            proprename = artist+" - "+album+" - "+tr+" - "+title+".mp3";
          }
          else if (!title.empty() && !artist.empty() && 
                   !album.empty() && track.empty())
          {
            proprename = artist+" - "+album+" - "+title+".mp3";
          }
          else if (!title.empty() && !artist.empty() && 
                   album.empty() && !track.empty())
          {
            proprename = artist+" - "+tr+" - "+title+".mp3";	
          }
          else if (!title.empty() && !artist.empty() && 
                   album.empty() && track.empty())
          {
            proprename = artist+" - "+title+".mp3";	
          }
        }
      }
    }
    catch(const FileSystemException& exp)
    {
      exp.showErrorDialog();
      no_selection();
      return;
    }
    w_file_prefs.set_sensitive(true);
    w_file_prefs.set_e_filename_text(s_last_file);
    w_file_prefs.set_e_filesize_text(file_size);
    w_file_prefs.set_e_prop_filename_text(proprename);
    w_tool_bar.b_delete_file_sensitive(true);
    w_tool_bar.b_rename_file_sensitive(true);
    w_menu_bar.set_rename_sensitive(true);
    w_menu_bar.set_delete_sensitive(true);
    w_file_prefs.set_e_tag_title_text(title);
    w_file_prefs.set_e_tag_artist_text(artist);
    w_file_prefs.set_e_tag_album_text(album);		
    w_file_prefs.set_e_tag_track_text(track);
    if(hastag)
    {
      w_tool_bar.b_tag_file_sensitive(true);
      w_tool_bar.b_save_all_sensitive(true);
      w_tool_bar.b_tag_add_file_sensitive(false);
      w_tool_bar.b_tag_del_file_sensitive(true);			
      w_menu_bar.set_tag_save_sensitive(true);
      w_menu_bar.set_save_all_sensitive(true);
      w_menu_bar.set_tag_add_sensitive(false);
      w_menu_bar.set_tag_del_sensitive(true);
      w_file_prefs.set_e_tags_sensitive(true);
      switch(hastag)
      {
        case 1: // ID3V1
                w_file_prefs.set_e_tag_version_text(_("ID3 version 1"));
                break;
        case 2: // ID3V2
                w_file_prefs.set_e_tag_version_text(_("ID3 version 2"));
                break;
        case 3: // ID3V1&2
                w_file_prefs.set_e_tag_version_text(_("ID3 version 1 and 2"));
                break;
      }
      if(!genre.empty())
        w_file_prefs.set_e_tag_genre_text(genre);
      else
        w_file_prefs.set_e_tag_genre_text("Unknown");
    }
    else{
      w_tool_bar.b_tag_add_file_sensitive(true);
      w_tool_bar.b_tag_del_file_sensitive(false);
      w_tool_bar.b_tag_file_sensitive(false);
      w_tool_bar.b_save_all_sensitive(false);
      w_menu_bar.set_tag_save_sensitive(false);
      w_menu_bar.set_save_all_sensitive(false);
      w_menu_bar.set_tag_add_sensitive(true);
      w_menu_bar.set_tag_del_sensitive(false);
      w_file_prefs.set_e_tags_sensitive(false);
      w_file_prefs.set_e_tag_version_text("");
      w_file_prefs.set_e_tag_genre_text("Unknown");
    }
  }
}

void WindowToplevel::s_button_rename_clicked()
{
  // check if input empty
  if(w_file_prefs.get_e_prop_filename_text().empty())
  {
    Gtk::MessageDialog d_warn(Glib::locale_to_utf8(_("No filename specified!")),
                              Gtk::MESSAGE_ERROR, Gtk::BUTTONS_CLOSE);
    d_warn.run();
  }
  // check if file will be renamed to the same name it already has
  else if(w_file_prefs.get_e_prop_filename_text() == 
          w_file_browse.getCurrentRowContent())
  {
    Gtk::MessageDialog d_warn(Glib::locale_to_utf8(
                              _("File already has the name"))+" \""+
                              w_file_prefs.get_e_prop_filename_text()+"\" "+
                              Glib::locale_to_utf8(_("and won't be renamed!")),
                              Gtk::MESSAGE_INFO, Gtk::BUTTONS_CLOSE);
    d_warn.run();
  }
  else
  {
    bool check = false;
    // check if file already exists
    if( w_file_browse.searchName(w_file_prefs.get_e_prop_filename_text())  )
    {
      Gtk::MessageDialog d_query(Glib::locale_to_utf8(
                        _("File"))+" \""+
                        w_file_prefs.get_e_prop_filename_text()+"\" "+
                        Glib::locale_to_utf8(_("already exists!\nOverwrite?")),
                        Gtk::MESSAGE_WARNING, Gtk::BUTTONS_YES_NO);
      int result = d_query.run();
      if(result == Gtk::RESPONSE_NO)
        return;
      else
        check = true;
    }	
    try{
      Gtk::TreeModel::iterator it;
      if(check)
        it = w_file_browse.getName(w_file_prefs.get_e_prop_filename_text());			
      if(rename(Glib::filename_from_utf8(s_current_dir+s_last_file).c_str(),
         Glib::filename_from_utf8(s_current_dir+
         w_file_prefs.get_e_prop_filename_text()).c_str()))
        throw(FileSystemException(FileSystemException::FS_RENAME, 
                                  Glib::locale_to_utf8(strerror(errno))));
      if(check){
        w_file_browse.del_row(it);
        set_files(all_files-1);
      }
      w_file_browse.rename_selected_row(
                    w_file_prefs.get_e_prop_filename_text());
      /* w_statusbar.set_text(" "+Glib::locale_to_utf8(
                             _("File renamed successfully")),0,8000); */
      reload_properties();
    }
    catch(const FileSystemException& exp)
    {
      exp.showErrorDialog();
    }
    catch(...)
    {
      ErrorDialog d_error(Glib::locale_to_utf8(
                          _("An unknown error has occured.")));
      d_error.run();
    }
  }
}

void WindowToplevel::s_button_delete_clicked()
{
  Gtk::MessageDialog d_query(Glib::locale_to_utf8(
       _("Do you really want to delete the file"))+" \""+
       w_file_browse.getCurrentRowContent()+"\""+Glib::locale_to_utf8(_(" ?")),
       Gtk::MESSAGE_QUESTION, Gtk::BUTTONS_YES_NO);
  int result = d_query.run();
  switch(result)
  {
    case Gtk::RESPONSE_YES:
    {
      try
      {
        if(unlink(Glib::filename_from_utf8(s_current_dir+s_last_file).c_str()))
          throw(FileSystemException(FileSystemException::FS_DELETE, 
                                    Glib::locale_to_utf8(strerror(errno))));
        w_file_browse.del_selected_row();
        set_files(all_files-1);
        w_statusbar.set_text(" "+Glib::locale_to_utf8(
                             _("File deleted successfully")),0,8000);
        no_selection();
      }
      catch(const FileSystemException& exp)
      {
        no_selection();
        exp.showErrorDialog();
        return;
      }
      break;
    }
    default: break;
  }
}

void WindowToplevel::s_button_tag_add_clicked()
{
  bool check = false;
  do
  {
    TagAdder d_tag(Glib::locale_to_utf8(_("Create new ID3Tag...")), 
                   s_last_file);
    int result = d_tag.run();
    if(result == Gtk::RESPONSE_OK)
    {
      // check if empty input fields
      if(d_tag.getTitle().empty()  &&
         d_tag.getArtist().empty() &&
         d_tag.getAlbum().empty()  &&
         d_tag.getGenre() == "Unknown"  &&
         !d_tag.getTrack())
      {
        Gtk::MessageDialog d_msg(Glib::locale_to_utf8(
             _("You haven't specified any tag information!")),
             Gtk::MESSAGE_ERROR,Gtk::BUTTONS_CLOSE);
        d_msg.run();
        continue;
      }
      else
      {
        // check if file still exists
        struct stat attribut;
        try
        {
          if(lstat(Glib::filename_from_utf8(s_current_dir+
             w_file_browse.getCurrentRowContent()).c_str(), &attribut) == -1)
            throw(FileSystemException(FileSystemException::FS_STAT_FILE,
                                      Glib::locale_to_utf8(strerror(errno))));
          check = true;
          ID3_Tag Tag;
          Tag.Clear();
          Tag.Link(Glib::filename_from_utf8(s_current_dir+
                   w_file_browse.getCurrentRowContent()).c_str());
          ID3_AddTitle(&Tag, locale_from_utf8(d_tag.getTitle()).c_str(), true);
          ID3_AddArtist(&Tag, locale_from_utf8(d_tag.getArtist()).c_str(),true);
          ID3_AddAlbum(&Tag, locale_from_utf8(d_tag.getAlbum()).c_str(), true);
          ID3_AddTrack(&Tag, d_tag.getTrack(),0, true);
          if(d_tag.getGenre() != "Unknown")
          {
            std::string s(d_tag.getGenre());
            for(uint i=0; i<148;i++)
              if(s == genre_table[i])
                ID3_AddGenre(&Tag,i);
          }
          if(s_current_tag_version == "id3v2")
          {
            Tag.Update(ID3TT_ID3V2);
            w_statusbar.set_text(Glib::locale_to_utf8(" ")+
                                 Glib::locale_to_utf8(
                                 _("ID3v2 Tag created successfully")),0,8000);
          }
          else if(s_current_tag_version == "id3v1")
          {
            Tag.Update(ID3TT_ID3V1);
            w_statusbar.set_text(Glib::locale_to_utf8(" ")+
                                 Glib::locale_to_utf8(
                                 _("ID3v1 Tag created successfully")),0,8000);
          }
          else if(s_current_tag_version == "id3v12")
          {
            Tag.Update(ID3TT_ID3V1);
            Tag.Update(ID3TT_ID3V2);
            w_statusbar.set_text(Glib::locale_to_utf8(" ")+
                                 Glib::locale_to_utf8(
                                 _("ID3v1&2 Tag created successfully")),0,8000);
          }
          reload_properties();
        }
        catch(const FileSystemException& exc)
        {
          no_selection();
          exc.showErrorDialog();
          return;
        }
      }
    }
    else
      check = true;
  }
  while(!check);
}

void WindowToplevel::s_button_tag_del_clicked()
{
  Gtk::MessageDialog d_query(Glib::locale_to_utf8(
                             _("Do you really want to delete this ID3Tag?")), 
                             Gtk::MESSAGE_QUESTION, Gtk::BUTTONS_YES_NO);
  int result = d_query.run();
  if(result == Gtk::RESPONSE_YES)
  {
    ID3_Tag Tag;
    unsigned long int tags;
    Tag.Clear();
    Tag.Link(Glib::filename_from_utf8(s_current_dir+
	           w_file_browse.getCurrentRowContent()).c_str());
    tags = Tag.Strip();
    w_statusbar.set_text(Glib::locale_to_utf8(" ")+
	        Glib::locale_to_utf8(_("Tag(s) removed successfully")),0,8000);
    reload_properties();
  }
}

void WindowToplevel::s_button_tag_clicked()
{
  // check if file still exists
  struct stat attribut;
  try
  {
    if(lstat(Glib::filename_from_utf8(s_current_dir+
             w_file_browse.getCurrentRowContent()).c_str(), &attribut) == -1)
      throw(FileSystemException(FileSystemException::FS_STAT_FILE,
                                Glib::locale_to_utf8(strerror(errno))));	
    ID3_Tag Tag;
    Tag.Clear();
    Tag.Link(Glib::filename_from_utf8(s_current_dir+
                   w_file_browse.getCurrentRowContent()).c_str());
    int conv = 0;
    if(Tag.HasV1Tag())
      conv=conv+1;
    if(Tag.HasV2Tag())
      conv=conv+2;
    // Clear ALL Tagging informations
    Tag.Strip();
    Tag.Clear();
    Tag.Link(Glib::filename_from_utf8(s_current_dir+
                   w_file_browse.getCurrentRowContent()).c_str());
    if(!w_file_prefs.get_e_tag_title_text().empty())
      ID3_AddTitle(&Tag, locale_from_utf8(
                   w_file_prefs.get_e_tag_title_text()).c_str(), true);
    if(!w_file_prefs.get_e_tag_artist_text().empty())
      ID3_AddArtist(&Tag, locale_from_utf8(
                    w_file_prefs.get_e_tag_artist_text()).c_str(), true);
    if(!w_file_prefs.get_e_tag_album_text().empty())
      ID3_AddAlbum(&Tag, locale_from_utf8(
                   w_file_prefs.get_e_tag_album_text()).c_str(), true);
    if(w_file_prefs.get_e_tag_track_text())
      ID3_AddTrack(&Tag, w_file_prefs.get_e_tag_track_text() ,0, true);
    if(w_file_prefs.get_e_tag_genre_text() != "Unknown")
    {
      std::string s(w_file_prefs.get_e_tag_genre_text());
      for(uint i=0; i < 148; i++)
      {
        if(s == genre_table[i])
          ID3_AddGenre(&Tag,i);
      }
    }
    if(s_current_tag_version == "id3v2"){
      Tag.Update(ID3TT_ID3V2);
    }
    else if(s_current_tag_version == "id3v1")
    {
        Tag.Update(ID3TT_ID3V1);
    }
    else if(s_current_tag_version == "id3v12")
    {
      Tag.Update(ID3TT_ID3V1);
      Tag.Update(ID3TT_ID3V2);
    }
    if(conv==1)
    {
      if(s_current_tag_version == "id3v1")
        w_statusbar.set_text(Glib::locale_to_utf8(" ")+
                             Glib::locale_to_utf8(
                             _("ID3v1 Tag saved successfully")),0,8000);
      else
        w_statusbar.set_text(Glib::locale_to_utf8(" ")+
                             Glib::locale_to_utf8(
                             _("ID3v1 Tag converted successfully")),0,8000);
    }
    else if(conv==2)
    {
      if(s_current_tag_version == "id3v2")
        w_statusbar.set_text(Glib::locale_to_utf8(" ")+
                             Glib::locale_to_utf8(
                             _("ID3v2 Tag saved successfully")),0,8000);
      else
        w_statusbar.set_text(Glib::locale_to_utf8(" ")+
                             Glib::locale_to_utf8(
                             _("ID3v2 Tag converted successfully")),0,8000);
    }
    else if(conv==3)
    {
      if(s_current_tag_version == "id3v12")
        w_statusbar.set_text(Glib::locale_to_utf8(" ")+
                             Glib::locale_to_utf8(
                             _("ID3v1&2 Tag saved successfully")),0,8000);
      else
        w_statusbar.set_text(Glib::locale_to_utf8(" ")+
                             Glib::locale_to_utf8(
                             _("ID3v1&2 Tag converted successfully")),0,8000);
    }
    reload_properties();
  }
  catch(const FileSystemException& exc)
  {
    no_selection();
    exc.showErrorDialog();
  }
}
	
void WindowToplevel::no_selection()
{
  s_last_file = "";
  w_file_prefs.reset_all_entries();
  w_file_prefs.set_sensitive(false);
  w_tool_bar.b_delete_file_sensitive(false);
  w_tool_bar.b_rename_file_sensitive(false);
  w_tool_bar.b_tag_file_sensitive(false);
  w_tool_bar.b_tag_add_file_sensitive(false);
  w_tool_bar.b_tag_del_file_sensitive(false);
  w_tool_bar.b_save_all_sensitive(false);
  w_menu_bar.set_rename_sensitive(false);
  w_menu_bar.set_delete_sensitive(false);
  w_menu_bar.set_tag_save_sensitive(false);
  w_menu_bar.set_tag_add_sensitive(false);
  w_menu_bar.set_tag_del_sensitive(false);
  w_menu_bar.set_save_all_sensitive(false);
}

// source == 0 (Menu), 1 (Toolbar)
void WindowToplevel::set_tag_version(int source)
{
  if(source)
  {
    int v = w_tool_bar.get_version_button();
    switch(v)
    {
      case 1: s_current_tag_version = "id3v1";
              w_menu_bar.set_version_button(1);
              break;
      case 2: s_current_tag_version = "id3v2";
              w_menu_bar.set_version_button(2);
              break;
      case 3: s_current_tag_version = "id3v12";
              w_menu_bar.set_version_button(3);
              break;
      default: break;
    }
  }
  else
  {
    int v = w_menu_bar.get_version_button();
    switch(v)
    {
      case 1: s_current_tag_version = "id3v1";
              w_tool_bar.set_version_button(1);
              break;
      case 2: s_current_tag_version = "id3v2";
              w_tool_bar.set_version_button(2);
              break;
      case 3: s_current_tag_version = "id3v12";
              w_tool_bar.set_version_button(3);
              break;
      default: break;
    }
  }
}

void WindowToplevel::s_reload_pressed()
{
  struct stat attribut;
  try
  {
    if(lstat(Glib::filename_from_utf8(s_current_dir).c_str(), &attribut) == -1)
      throw(FileSystemException(FileSystemException::FS_STAT_DIR,
                                Glib::locale_to_utf8(strerror(errno))));
    if (S_ISDIR(attribut.st_mode))
    {
      DIR *p_dir;
      if( (p_dir = opendir(Glib::filename_from_utf8(s_current_dir).c_str())) 
	   == NULL)
      {
        closedir(p_dir);
        throw(FileSystemException(FileSystemException::FS_STAT_DIR));
      }
      w_file_browse.clear();
      
      struct dirent *p_dirent;
      std::vector<Glib::ustring> entries;
      struct stat att;
      unsigned long files=0;
      
      while( (p_dirent = readdir(p_dir)) != NULL)
      {
        if(strcmp(p_dirent->d_name,".") && strcmp(p_dirent->d_name, ".."))
        {
          Glib::ustring tmp = Glib::filename_to_utf8(p_dirent->d_name);
          if(lstat(Glib::filename_from_utf8(s_current_dir+tmp).c_str(), &att) 
             != -1)
          {
            if (S_ISREG(att.st_mode))
            { 
              entries.push_back(Glib::filename_to_utf8(p_dirent->d_name));
              files++;
            }
          }
        }
      }
      closedir(p_dir);
      std::stable_sort(entries.begin(),entries.end());
      for(uint i=0; i<entries.size(); i++)
      {
        if(entries[i].size() > 2)
        {
          Glib::ustring s(entries[i],entries[i].size()-3,3);
          if(     (s == "mp3") || (s == "mP3") || (s == "Mp3") || (s == "MP3")
              ||  (s == "ogg") || (s == "Ogg") || (s == "OGg") || (s == "OgG")
              ||  (s == "oGG") || (s == "ogG") || (s == "oGg")) 
            w_file_browse.appendRow(entries[i], FileBrowse::SOUND_MIME);
          else
            w_file_browse.appendRow(entries[i], FileBrowse::UNKNOWN_MIME);
        }
        else
          w_file_browse.appendRow(entries[i], FileBrowse::UNKNOWN_MIME);								
      }
      set_files(files);
      w_statusbar.set_text(Glib::locale_to_utf8(" ")+
                           Glib::locale_to_utf8(
                           _("List reloaded successfully")),0,8000);
      no_selection();
    }
  }
  catch(const FileSystemException& exp)
  {
    exp.showErrorDialog();
  }
  catch(...)
  {
    ErrorDialog d_error(Glib::locale_to_utf8(
                        _("An unknown error has occured.")));
    d_error.run();
  }	
}

void WindowToplevel::set_files(unsigned long f)
{
  all_files = f;
  std::ostringstream os;
  os << " " << f << " ";
  if(f==1)
    w_statusbar.set_text(os.str()+Glib::locale_to_utf8(_("File")),1);
  else 
    w_statusbar.set_text(os.str()+Glib::locale_to_utf8(_("Files")),1);
}

bool WindowToplevel::tag_changed(GdkEventKey* event)
{
  refresh_proposal_filename();
  return true;	
}

void WindowToplevel::refresh_proposal_filename()
{
  Glib::ustring title = w_file_prefs.get_e_tag_title_text();
  Glib::ustring artist = w_file_prefs.get_e_tag_artist_text();
  Glib::ustring album = w_file_prefs.get_e_tag_album_text();
  std::ostringstream tr;
  if(w_file_prefs.get_e_tag_track_text())
    tr << static_cast<unsigned int>(w_file_prefs.get_e_tag_track_text());
  Glib::ustring track = tr.str();
  Glib::ustring proprename;
  if(track.size()==1)
    track = "0"+track;
  if (!title.empty() && !artist.empty() && !album.empty() && !track.empty())
  {
    proprename = artist+" - "+album+" - "+track+" - "+title+".mp3";
  }
  else if (!title.empty() && !artist.empty() && !album.empty() && track.empty())
  {
    proprename = artist+" - "+album+" - "+title+".mp3";
  }
  else if (!title.empty() && !artist.empty() && album.empty() && !track.empty())
  {
    proprename = artist+" - "+track+" - "+title+".mp3";	
  }
  else if (!title.empty() && !artist.empty() && album.empty() && track.empty())
  {
    proprename = artist+" - "+title+".mp3";	
  }
  w_file_prefs.set_e_prop_filename_text(proprename);
}

void WindowToplevel::s_save_all_pressed()
{
  /* RENAME */
  
  bool fexists = false; // file already exists
  bool check = false; // overwrite evtl. existing file?
	
  // check if input empty
  if(w_file_prefs.get_e_prop_filename_text().empty())
  {
    Gtk::MessageDialog d_warn(Glib::locale_to_utf8(_("No filename specified!")),
                              Gtk::MESSAGE_ERROR, Gtk::BUTTONS_CLOSE);
    d_warn.run();
  }
  // check if file will be renamed to the same name it already has
  else if(w_file_prefs.get_e_prop_filename_text() == 
          w_file_browse.getCurrentRowContent())
  {
    Gtk::MessageDialog d_warn(Glib::locale_to_utf8(
                              _("File already has the name"))+" \""+
                              w_file_prefs.get_e_prop_filename_text()+"\" "+
                              Glib::locale_to_utf8(_("and won't be renamed!")),
                              Gtk::MESSAGE_INFO, Gtk::BUTTONS_CLOSE);
    d_warn.run();
  }
  else
  {
    // check if file already exists
    if( w_file_browse.searchName(w_file_prefs.get_e_prop_filename_text())  )
    {
      Gtk::MessageDialog d_query(Glib::locale_to_utf8(
                        _("File"))+" \""+
                        w_file_prefs.get_e_prop_filename_text()+"\" "+
                        Glib::locale_to_utf8(_("already exists!\nOverwrite?")),
                        Gtk::MESSAGE_WARNING, Gtk::BUTTONS_YES_NO);
      int result = d_query.run();
      fexists = true;
      if(result == Gtk::RESPONSE_NO)
        return;
      else
        check = true;
    }	
  }
  try
  {
    Gtk::TreeModel::iterator it;
    if(fexists && check)
      it = w_file_browse.getName(w_file_prefs.get_e_prop_filename_text());
    if(!fexists)
    {
      if(rename(Glib::filename_from_utf8(s_current_dir+s_last_file).c_str(),
         Glib::filename_from_utf8(s_current_dir+
         w_file_prefs.get_e_prop_filename_text()).c_str()))
        throw(FileSystemException(FileSystemException::FS_RENAME, 
                                  Glib::locale_to_utf8(strerror(errno))));
    }
    else if(check)
    {
      if(rename(Glib::filename_from_utf8(s_current_dir+s_last_file).c_str(),
         Glib::filename_from_utf8(s_current_dir+
         w_file_prefs.get_e_prop_filename_text()).c_str()))
        throw(FileSystemException(FileSystemException::FS_RENAME, 
                                  Glib::locale_to_utf8(strerror(errno))));
      w_file_browse.del_row(it);
      set_files(all_files-1);
    }
    w_file_browse.rename_selected_row(
                  w_file_prefs.get_e_prop_filename_text());

    /* SAVE TAGS */
    // check if file still exists
    struct stat attribut;
    if(lstat(Glib::filename_from_utf8(s_current_dir+
             w_file_browse.getCurrentRowContent()).c_str(), &attribut) 
             == -1)
      throw(FileSystemException(FileSystemException::FS_STAT_FILE,
                            Glib::locale_to_utf8(strerror(errno))));	
    ID3_Tag Tag;
    Tag.Clear();
    Tag.Link(Glib::filename_from_utf8(s_current_dir+
                   w_file_browse.getCurrentRowContent()).c_str());
    int conv = 0;
    if(Tag.HasV1Tag())
      conv=conv+1;
    if(Tag.HasV2Tag())
      conv=conv+2;
    // Clear ALL Tagging informations
    Tag.Strip();
    Tag.Clear();
    Tag.Link(Glib::filename_from_utf8(s_current_dir+
                   w_file_browse.getCurrentRowContent()).c_str());
    if(!w_file_prefs.get_e_tag_title_text().empty())
      ID3_AddTitle(&Tag, locale_from_utf8(
                   w_file_prefs.get_e_tag_title_text()).c_str(), true);
    if(!w_file_prefs.get_e_tag_artist_text().empty())
      ID3_AddArtist(&Tag, locale_from_utf8(
                    w_file_prefs.get_e_tag_artist_text()).c_str(), true);
    if(!w_file_prefs.get_e_tag_album_text().empty())
      ID3_AddAlbum(&Tag, locale_from_utf8(
                   w_file_prefs.get_e_tag_album_text()).c_str(), true);
    if(w_file_prefs.get_e_tag_track_text())
      ID3_AddTrack(&Tag, w_file_prefs.get_e_tag_track_text() ,0, true);
    if(w_file_prefs.get_e_tag_genre_text() != "Unknown")
    {
      std::string s(w_file_prefs.get_e_tag_genre_text());
      for(uint i=0; i < 148; i++)
      {
        if(s == genre_table[i])
          ID3_AddGenre(&Tag,i);
      }
    }
    if(s_current_tag_version == "id3v2")
    {
      Tag.Update(ID3TT_ID3V2);
    }
    else if(s_current_tag_version == "id3v1")
    {
      Tag.Update(ID3TT_ID3V1);
    }
    else if(s_current_tag_version == "id3v12")
    {
      Tag.Update(ID3TT_ID3V1);
      Tag.Update(ID3TT_ID3V2);
    }
    if(conv==1)
    {
      if(s_current_tag_version == "id3v1")
        w_statusbar.set_text(Glib::locale_to_utf8(" ")+
              Glib::locale_to_utf8(
              _("File renamed & ID3v1 Tag saved successfully")),0,8000);
      else
        w_statusbar.set_text(Glib::locale_to_utf8(" ")+
               Glib::locale_to_utf8(
              _("File renamed & ID3v1 Tag converted successfully")),0,8000);
    }
    else if(conv==2)
    {
      if(s_current_tag_version == "id3v2")
        w_statusbar.set_text(Glib::locale_to_utf8(" ")+
             Glib::locale_to_utf8(
             _("File renamed & ID3v2 Tag saved successfully")),0,8000);
      else
        w_statusbar.set_text(Glib::locale_to_utf8(" ")+
             Glib::locale_to_utf8(
             _("File renamed & ID3v2 Tag converted successfully")),0,8000);
    }
    else if(conv==3)
    {
      if(s_current_tag_version == "id3v12")
        w_statusbar.set_text(Glib::locale_to_utf8(" ")+
          Glib::locale_to_utf8(
          _("File renamed & ID3v1&2 Tag saved successfully")),0,8000);
      else
        w_statusbar.set_text(Glib::locale_to_utf8(" ")+
          Glib::locale_to_utf8(
          _("File renamed & ID3v1&2 Tag converted successfully")),0,8000);
    }    
  }
  catch(const FileSystemException& exp)
  {
    no_selection();
    exp.showErrorDialog();
  }
  catch(...)
  {
    ErrorDialog d_error(Glib::locale_to_utf8(
                        _("An unknown error has occured.")));
    d_error.run();
  }
  reload_properties();
}
